/*
 * Brocade platform support for the 440GP based products.
 * Based on:  arch/ppc/platforms/ebony.c - Copyright 2002 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */
#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blkdev.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/ide.h>
#include <linux/initrd.h>
#include <linux/irq.h>
#include <linux/seq_file.h>
#include <linux/root_dev.h>
#include <linux/tty.h>
#include <linux/serial.h>
#include <linux/serial_core.h>
#include <brocade/cpld.h>
#include <linux/ethtool.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/ocp.h>
#include <asm/pci-bridge.h>
#include <asm/time.h>
#include <asm/todc.h>
#include <asm/bootinfo.h>
#include <asm/ppc4xx_pic.h>
#include <syslib/gen550.h>
#include <brocade/platform.h>
#include <brocade/silkworm.h>
#include <brocade/silkworm_features.h>
#include "silkworm_cpld.h"
/*
#include <asm/ibm440.h>
#include <platforms/4xx/silkworm440.h>
*/

static struct ibm44x_clocks clocks __initdata;

/* Base address for the PCIX configuration registers */
int *pcix_reg_base;

extern sw_bd silkworm;
extern void ppc4xx_wdt_heartbeat(void);
extern int wdt_enable;
extern unsigned long wdt_period;

#ifndef CONFIG_SILKWORM440_AP
extern int cpld_init(void);
#endif
extern void gen550_progress(char *, unsigned short);

static void ppc440_restart(char *cmd);
static void ppc440_shutdown(void);

unsigned char *ppc4xx_uic_ext_irq_cfg __initdata = NULL;

/*
 * IRQ triggering/polarity settings
 */
static u_char ppc440_IRQ_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 23: Ext Int 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 24: Ext Int 1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 25: Ext Int 2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 26: Ext Int 3 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 27: Ext Int 4 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 28: Ext Int 5 - CompactFlash */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 29: Ext Int 6 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 50: Ext Int 7 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 51: Ext Int 8 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 52: Ext Int 9 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 53: Ext Int 10 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 54: Ext Int 11 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 55: Ext Int 12 */
};

/* Sensitive Table for Pulsar external interrupts */
static u_char ppc440pulsar_IRQ_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* 23: IRQ0, PCI slot 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* 24: IRQ1, PCI slot 1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* 25: IRQ2, PCI slot 2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* 26: IRQ3, PCI slot 3 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 27: IRQ4, IRDA */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* 28: IRQ5, SMI pushbutton */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* 29: IRQ6, PHYs */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 50: IRQ7, AUX */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* 51: IRQ8, EXT */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* 52: IRQ9, EXT */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* 53: IRQ10, EXT */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 54: IRQ11, EXT */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 55: IRQ12, EXT */
};


/* Sensitive Table for Stealth */
static u_char ppc440stealth_IRQ_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 23: Ext Int 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 24: Ext Int 1 - GoldenEye */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 25: Ext Int 2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 26: Ext Int 3 - Sys Ints */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 27: Ext Int 4 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 28: Ext Int 5 - Compact Flash */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 29: Ext Int 6 - Mictor connector */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 50: Ext Int 7 - Ethernet phy */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 51: Ext Int 8 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 52: Ext Int 9 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 53: Ext Int 10 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 54: Ext Int 11 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 55: Ext Int 12 */
};


/* Sensitive Table for Viking */
static u_char ppc440viking_IRQ_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 23: Ext Int 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 24: Ext Int 1 - Condor */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 25: Ext Int 2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 26: Ext Int 3 - SYS_ATTN_INT */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 27: Ext Int 4 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 28: Ext Int 5 - CompactFlash */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 29: Ext Int 6 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 50: Ext Int 7 - ENET_PHY0_INT */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 51: Ext Int 8 - ENET_PHY0_INT(resv) */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 52: Ext Int 9 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 53: Ext Int 10 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 54: Ext Int 11 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 55: Ext Int 12 */
};


/* Sensitive Table for  Blazer2 */
static u_char ppc440blazer2_IRQ_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 23: Ext Int 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 24: Ext Int 1 - GoldenEye */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 25: Ext Int 2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 26: Ext Int 3 - Sys Ints */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 27: Ext Int 4 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 28: Ext Int 5 - Compact Flash */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 29: Ext Int 6 - Mictor connector */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 50: Ext Int 7 - Ethernet phy */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 51: Ext Int 8 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 52: Ext Int 9 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 53: Ext Int 10 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 54: Ext Int 11 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 55: Ext Int 12 */
};


/* Sensitive Table for  glacier2 */
static u_char ppc440glacier2_IRQ_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 23: Ext Int 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 24: Ext Int 1 - GoldenEye */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 25: Ext Int 2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 26: Ext Int 3 - Sys Ints */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 27: Ext Int 4 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 28: Ext Int 5 - Compact Flash */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 29: Ext Int 6 - Mictor connector */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 50: Ext Int 7 - Ethernet phy */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 51: Ext Int 8 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 52: Ext Int 9 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 53: Ext Int 10 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 54: Ext Int 11 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 55: Ext Int 12 */
};


/* Sensitive Table for LittleBear 8G external interrupts */
static u_char ppc440littlebear8g_IRQ_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* 23: Ext IRQ0, pull down */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 24: Ext IRQ1, GoldenEye 2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 25: Ext IRQ2, pull high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 26: Ext IRQ3, System Attention */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 27: Ext IRQ4, pull high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* 28: Ext IRQ5, Compact Flash */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 29: Ext IRQ6, pull high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 50: Ext IRQ7, pull high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 51: Ext IRQ8, pull high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 52: Ext IRQ9, pull high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 53: Ext IRQ10, pull high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 54: Ext IRQ11, pull high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),  /* 55: Ext IRQ12, pull high */
};

extern void abort(void);

/* Global Variables */
unsigned char __res[sizeof (bd_t)];
unsigned long serial_base_baud;
static char serial_clock_external_flag;

static unsigned long tbh;
static int __init tbh_setup(char *str)
{
    tbh = simple_strtoul(str, NULL, 0);
	printk(KERN_NOTICE "Setting tbh to %lx\n", tbh);
    return 1;
}
__setup("tbh=", tbh_setup);

static void __init
ppc440_calibrate_decr(void)
{
	unsigned int freq;
#ifdef CONFIG_SILKWORM440
	bd_t *bip = (bd_t *) __res;
	freq = bip->bi_tbfreq;
#else

	/*
	 * Determine system clock speed
	 *
	 * If we are on Rev. B silicon, then use
	 * default external system clock.  If we are
	 * on Rev. C silicon then errata forces us to
	 * use the internal clock.
	 */
	switch (PVR_REV(mfspr(PVR))) {
		case PVR_REV(PVR_440GP_RB):
			freq = SILKWORM_440GP_RB_SYSCLK;
			break;
		case PVR_REV(PVR_440GP_RC1):
		default:
			freq = SILKWORM_440GP_RC_SYSCLK;
			break;
	}
#endif
	ibm44x_calibrate_decr(freq);
}

/*
 *   This routine pretty-prints the platform's internal bus clock
 *   frequencies into the buffer for usage in /proc/cpuinfo.
 */
static int
ppc440_show_cpuinfo(struct seq_file *m)
{
	bd_t *bip = (bd_t *) __res;

	seq_printf(m, "machine\t\t: %s\n", PPC440_MACHINE_NAME);
	seq_printf(m, "plb bus clock\t: %ldMHz\n",
		   (long) bip->bi_busfreq / 1000000);
#ifdef CONFIG_PCI
	seq_printf(m, "pci bus clock\t: %dMHz\n",
		   bip->bi_pci_busfreq / 1000000);
#endif
	seq_printf(m, "uart clock\t: %luKHz %s\n",
		   serial_base_baud / 1000,
		   ((serial_clock_external_flag) ? "(external)" : "(internal)"));
	return 0;
}


static inline int
ppc440_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
#ifdef CONFIG_SILKWORM
	/* Boot PROM takes care of IRQ routing for Silkworm family */
	int irq;

	if(dev)
		irq = dev->irq;
	else
		irq = -1;
	return irq;
#else
	static char pci_irq_table[][4] =
	/*
	 *	PCI IDSEL/INTPIN->INTLINE 
	 * 	   A   B   C   D
	 */
	{
		{ 23, 23, 23, 23 },	/* IDSEL 1 - PCI Slot 0 */
		{ 24, 24, 24, 24 },	/* IDSEL 2 - PCI Slot 1 */
		{ 25, 25, 25, 25 },	/* IDSEL 3 - PCI Slot 2 */
		{ 26, 26, 26, 26 },	/* IDSEL 4 - PCI Slot 3 */
	};

	const long min_idsel = 1, max_idsel = 4, irqs_per_slot = 4;
	return PCI_IRQ_TABLE_LOOKUP;
#endif
}

#define PCIX_WRITEL(value, offset) \
	(writel(value, pcix_reg_base + offset))
#define PCIX_READL(offset) \
        (readl(pcix_reg_base + offset))

/*
 * FIXME: This is only here to "make it work".  This will move
 * to a ibm_pcix.c which will contain a generic IBM PCIX bridge
 * configuration library. -Matt
 */
static void __init
ppc440_setup_pcix(void)
{
        unsigned int reg_val;
	pcix_reg_base = ioremap64(PCIX0_REG_BASE, PCIX0_REG_SIZE);

        /*
         * The PPC440 PCI host bridge has a default CLASS value of
         * PCI_CLASS_BRIDGE_OTHER.  This is messing up the scan code.
         * Make sure the 440 host bridge CLASS is PCI_CLASS_BRIDGE_HOST.
	 * N.B. The following WRITEL is only modifying a byte and assumes
	 * single threaded operation.
         */
        reg_val = PCIX_READL(PCIX0_REVID);
        reg_val = (reg_val & 0xFF) | (PCI_CLASS_BRIDGE_HOST << 16);
        PCIX_WRITEL(reg_val, PCIX0_REVID);
        eieio();
}

static void __init
ppc440_setup_hose(void)
{
	struct pci_controller *hose;

	/* Configure windows on the PCI-X host bridge */
	ppc440_setup_pcix();

	hose = pcibios_alloc_controller();

	if (!hose)
		return;

	hose->first_busno = 0;
	hose->last_busno = 0xff;

	hose->pci_mem_offset = PPC440_PCI_MEM_OFFSET;

	pci_init_resource(&hose->io_resource,
			PPC440_PCI_LOWER_IO,
			PPC440_PCI_UPPER_IO,
			IORESOURCE_IO,
			"PCI host bridge");

	pci_init_resource(&hose->mem_resources[0],
			PPC440_PCI_LOWER_MEM,
			PPC440_PCI_UPPER_MEM,
			IORESOURCE_MEM,
			"PCI host bridge");

	hose->io_space.start = PPC440_PCI_LOWER_IO;
	hose->io_space.end = PPC440_PCI_UPPER_IO;
	hose->mem_space.start = PPC440_PCI_LOWER_MEM;
	hose->mem_space.end = PPC440_PCI_UPPER_MEM;
	isa_io_base = ioremap64(PPC440_PCI_IO_BASE, PPC440_PCI_IO_SIZE);

	hose->io_base_virt = (void *)isa_io_base;

	/*
	 * setup_indirect_pci() doesn't support 36-bit real address. Thus,
	 * This call will not map the PCI addresses for PPC440. We need to
	 * call ioremap() explicitly to map these addresses.
	 */
	setup_indirect_pci(hose,
			PPC440_PCI_CFGA_PLB32,
			PPC440_PCI_CFGD_PLB32);
	hose->cfg_addr = (unsigned int *) ioremap(PPC440_PCI_CFGA_PLB32, 4);
	hose->cfg_data = (unsigned char *) ioremap(PPC440_PCI_CFGD_PLB32, 4);

	hose->set_cfg_type = 1;

	/* Don't know why. It will crash inside this function. As in ppc405,
	 * I comment it out for now.
	 */
#if 0
	hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);
#endif
	hose->last_busno = 0;

	ppc_md.pci_swizzle = common_swizzle;
	ppc_md.pci_map_irq = ppc440_map_irq;
#ifdef CONFIG_SILKWORM
	ppc_md.pci_exclude_device	= NULL;	/* needed for db module */
#else
#endif
}

#ifdef CONFIG_PPC_RTC
TODC_ALLOC();
#endif /* CONFIG_PPC_RTC */

static void __init
ppc440_early_serial_map(void)
{
	struct uart_port port;

	/* Setup ioremapped serial port access */
	memset(&port, 0, sizeof(port));
	port.membase = ioremap64(PPC440GP_UART1_ADDR, 8);
	port.irq = 1;
	port.uartclk = clocks.uart1;
	port.regshift = 0;
	port.iotype = SERIAL_IO_MEM;
	port.flags = ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST;
	port.line = 0;

	if (early_serial_setup(&port) != 0) {
		printk("Early serial init of port 0 failed\n");
	}

#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	/* Configure debug serial access */
	gen550_init(0, &port);
#endif

	port.membase = ioremap64(PPC440GP_UART0_ADDR, 8);
	port.irq = 0;
	port.uartclk = clocks.uart0;
	port.line = 1;

	if (early_serial_setup(&port) != 0) {
		printk("Early serial init of port 1 failed\n");
	}

#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	/* Configure debug serial access */
	gen550_init(1, &port);
#endif
}

#define PHY_ADDR_TO_IBM_EMAC_PHY_MAP(phy_addr) (~(1 << (phy_addr)))

static void __init
ppc440_setup_arch(void)
{
	struct ocp_def *def;
	struct ocp_func_emac_data *emacdata;
	bd_t *bip = (bd_t *) __res;

	/* Set mac_addr for each EMAC */
	def = ocp_get_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 0);
	emacdata = def->additions;
	emacdata->phy_map = PHY_ADDR_TO_IBM_EMAC_PHY_MAP(1);
	/* LittleBear8G emac0 is connected to phy with mii mode. */
	if (silkworm.board == SWBD70) {
		emacdata->phy_mode = PHY_MODE_MII;
	} else {
		emacdata->phy_mode = PHY_MODE_RMII;
	}
	memcpy(emacdata->mac_addr, &bip->bi_enetaddr[0][0], 6);

#if defined(CONFIG_SILKWORM) || defined (CONFIG_SILKWORM_COMMON)
#ifdef CONFIG_IP_PNP_BOOTP

        /* setup eth0 for manufacturing/netboot for eth0
         *
         * In the case of BOOTP support where the kernel uses
         * IP autoconfiguration support for OPS, some platforms
         * need to be 100 FULL no Autonegotiation on eth0 else
         * Code under CONFIG_IP_PNP_BOOTP and CONFIG_IP_PNP does not
         * work and BOOTP protocol does not execute
         * The observation was the interface (eth0) would not come up.
         *
         * The phy_feat_exc means exclude selected features. You will
         * notice that default excludes all except 100BaseT FULL and
         * that is the way that we force No Autoneg, 100 FULL
         */

        switch (silkworm.board) {

        case SWBD43: /* Bradley, for San Jose OPS */
        case SWBD61: /* Superhawk, for San Jose OPS */

                        emacdata->phy_feat_exc = SUPPORTED_Autoneg
                                | SUPPORTED_10baseT_Full
                                | SUPPORTED_10baseT_Half
                                | SUPPORTED_100baseT_Half;
                        printk(KERN_CRIT
                                "ppc440_setup_arch(): BOOTP support: eth0 Setting 100fullduplex No Autoneg for %d\n",
                                silkworm.board);
                        break;
        }
#endif
#endif

	/* LIttlebear8G emac0 uses mii mode, emac1 can't be used. */
	if (silkworm.board != SWBD70) {
		def = ocp_get_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 1);
		emacdata = def->additions;
		emacdata->phy_map = PHY_ADDR_TO_IBM_EMAC_PHY_MAP(2);
		emacdata->phy_mode = PHY_MODE_RMII;
	
		/*
		 * On Meteor (swbd21) the eth1 is connected to phy with
		 * 100mps, full duplex hard coded. So exclude rest of cases.
		 * See the bamboo.c for insipiration
		 */

		switch (silkworm.board) {
			case SWBD21:	/* Meteor */
				emacdata->phy_feat_exc = SUPPORTED_Autoneg| SUPPORTED_10baseT_Full 
					 | SUPPORTED_10baseT_Half| SUPPORTED_100baseT_Half;
			default:
				printk(KERN_WARNING
					"ppc440_setup_arch(): Not setting the 100MBFull for eth1 on this SWBD%d\n",
					silkworm.board);
				break;

		}
		memcpy(emacdata->mac_addr, &bip->bi_enetaddr[0][0], 6);
		emacdata->mac_addr[5] += 1;
	}

	/*
	 * Determine various clocks.
	 * To be completely correct we should get SysClk
	 * from FPGA, because it can be changed by on-board switches
	 * --ebs
	 */
	ibm440gp_get_clocks(&clocks, 33333333, 6 * 1843200);
	ocp_sys_info.opb_bus_freq = clocks.opb;

	/* init to some ~sane value until calibrate_delay() runs */
        loops_per_jiffy = 50000000/HZ;

	/* Setup PCI host bridge */
	ppc440_setup_hose();

#ifdef CONFIG_DUMMY_CONSOLE
	conswitchp = &dummy_con;
#endif

	ppc440_early_serial_map();

	switch (silkworm.board) {
		case SWBD21:	/* Meteor */
			ppc4xx_uic_ext_irq_cfg = ppc440_IRQ_initsenses;
			break;

		case SWBD23:	/* Pulsar */
		case SWBD32:	/* AND Pulsar-32 */
		case SWBD58:	/* AND Pulsar2 */
			ppc4xx_uic_ext_irq_cfg = ppc440pulsar_IRQ_initsenses;
			break;

		case SWBD34:	/* Stealth */
		case SWBD29:	/* Icecube */
		case SWBD43:	/* Icecube2 */
			ppc4xx_uic_ext_irq_cfg = ppc440stealth_IRQ_initsenses;
			break;

		case SWBD44:	/* Viking */
			ppc4xx_uic_ext_irq_cfg = ppc440viking_IRQ_initsenses;
			break;

		case SWBD37:	/* Blazer2 */
			ppc4xx_uic_ext_irq_cfg = ppc440blazer2_IRQ_initsenses;
			break;

		case SWBD45:	/* glacier2 */
		case SWBD51:	/* Dragon */
		case SWBD61:	/* Superhawk */
			ppc4xx_uic_ext_irq_cfg = ppc440glacier2_IRQ_initsenses;
			break;

		case SWBD70:	/* LittleBear 8G */
			ppc4xx_uic_ext_irq_cfg = ppc440littlebear8g_IRQ_initsenses;
			break;

		default:
			printk(KERN_CRIT "%s: unsupported platform SWBD%d.  "
				"Cannot init IRQ senses table.\n",
				__FUNCTION__, silkworm.board);
			break;
	}

#ifndef CONFIG_SILKWORM440_AP
	cpld_init();
#endif


#ifdef CONFIG_SILKWORM440_AP
	ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0);
#endif
	/* Identify the system */
	printk("Brocade Silkworm port (C) 2002 MontaVista Software, Inc. (source@mvista.com)\n");
}

/*
 * ppc440_meteor_restart()
 *
 * Description:
 *  Hardware specific restart routine  for Brocade Meteor
 *  systems. The CPLD must be left in an appropriate state
 *  to allow the bootprom to access the board ID prior to
 *  configuring chip selects.
 *
 *  After doing the CPLD configuration, abort() is called
 *  to re-start the CPU.
 *
 * Input(s):
 *   cmd - Optional string to be printed indicating what
 *         command led to this routine.
 *
 * Output(s):
 *  None
 *
 * Returns:
 *  Function does not return.
 *
 */
static void
ppc440_meteor_restart(char *cmd)
{
#ifndef CONFIG_SILKWORM440_AP
	volatile unsigned char cpld_data;
	volatile char *cpld_base, *cpld_miscread, *cpld_miscwrite, *cpld_i2c;

	/*
	 * defect 50680
	 * get cpld base address from new function since doing the
	 * ioremap() here like it used to will break sometimes in ISR
	 * context.
	 */
	cpld_base = cpld_get_base_addr();
	if (cpld_base) {

		cpld_miscread = cpld_base + MET_CP_MISC_READ;
		cpld_miscwrite = cpld_base + MET_CP_MISC_WRITE;
		cpld_i2c = cpld_base + MET_I2C_SELECT;

		cpld_data = readb(cpld_miscread);

		/*
		* Read Slot ID bit. If it's 1, the CP
		* is in slot 6. If it's 0, the CP is in
		* slot 5. The I2C address mux must be
		* set to the slot number to allow the
		* board ID to be read by the bootprom
		* prior to configuring the CPLD
		*
		*/
		if (cpld_data & MET_CP_CP_SLOT_ID)
			cpld_data = 6;
		else
			cpld_data = 5;

		writeb(cpld_data, cpld_i2c);

		/*
		* Assert Error
		*/
		
		cpld_data = readb(cpld_miscwrite);
		cpld_data |= MET_CP_SECTION_ERR_OUT;
		writeb(cpld_data, cpld_miscwrite);

	} else {
		printk(KERN_WARNING "ppc440_meteor_restart: cpld_base NULL\n");
	}
#endif

	abort();
}


/*
 * ppc440_pulsar_shutdown()
 *
 * Description:
 *  Hardware specific shutdown routine  for Brocade Pulsar
 *  systems. The Pulsar CPLD has the  capability  to power
 *  down the system.  Arm and detonate.
 *
 * Input(s):
 *  None
 *
 * Output(s):
 *  None
 *
 * Returns:
 *  Function does not return.
 *
 */
static void
ppc440_pulsar_shutdown(void)
{
#ifndef CONFIG_SILKWORM440_AP
	volatile char *cpld_base, *cpld_reset_enable, *cpld_reset_shutdown;
	
	cpld_base = (char *) ioremap(PULSAR_CPLD_BASE_ADDR, 32);
	if (cpld_base) {

		cpld_reset_enable = cpld_base + PULSAR_SYS_RESET_ENABLE_REG;
		cpld_reset_shutdown = cpld_base + PULSAR_SYS_RESET_SHUTDOWN_REG;

		/*
		* Arm for system shutdown.
		*/
		writeb(PULSAR_SYSTEM_SHUTDOWN_ENABLE, cpld_reset_enable);
		
		/*
		* Shutdown the system.
		*/
		writeb(PULSAR_SYSTEM_SHUTDOWN, cpld_reset_shutdown);

	} else {
		printk(KERN_CRIT "ppc44x_shutdown: ioremap failed\n");
	}
#endif

}


/*
 * ppc440_superhawk_reset()
 *
 * Description:
 *   Satisfies a Superhawk requirement. Sets a bit in the CPLD "CMC support"
 *   register. This bit tells the chassis management subsystem that some user
 *   has intentionally reset the switch.
 */
static void
ppc440_superhawk_reset(void)
{
	volatile char *cpldp = cpld_get_base_addr();
	if (cpldp) {
		enum { SWITCH_INTENDED_RESET = 0x10 };
		cpldp += SUPERHAWK_CMC_SUP_REG;
		writeb(SWITCH_INTENDED_RESET | readb(cpldp), cpldp);
	} else {
		printk(KERN_WARNING "%s: cpld base is NULL\n", __FUNCTION__);
	}
}


static void
ppc440_restart(char *cmd)
{
	printk("%s\n", cmd);
	switch (silkworm.board) {
		case SWBD21:	/* Meteor */
			ppc440_meteor_restart(cmd);
			break;
		case SWBD61:
			ppc440_superhawk_reset();
			/* fall through to default */
		/* Default is abort */
		default:
			abort();
			break;
	}
}


static void 
ppc440_shutdown(void)
{
	cli();
	while (1) {
	  ppc405wd_update_timer_shutdown();

	}

}

static void
ppc440_power_off(void)
{
	switch (silkworm.board) {
		case SWBD23:	/* Pulsar */
		case SWBD32:	/* AND Pulsar-32 */
		case SWBD58:	/* AND Pulsar2 */
		case SWBD34:	/* AND Stealth */
		case SWBD44:	/* AND Viking */
			cpld_power_off();
			break;
		case SWBD61:
			ppc440_superhawk_reset();
			/* fall through to default */
		/* Default is restart routine */
		case SWBD29:	/* No shutdown capability on Icecube */
		default:
			ppc440_shutdown();
			break;
	}
}

static void
ppc440_halt(void)
{
	ppc440_restart(NULL);
}


/*
 * Read the 440GP memory controller to get size of system memory.
 */
static unsigned long __init
ppc440_find_end_of_memory(void)
{
#ifdef CONFIG_SILKWORM
	bd_t *bip = (bd_t *) __res;

	return ((unsigned long) bip->bi_memsize);
#else
	u32 i, bank_config;
	u32 mem_size = 0;

	for (i=0; i<4; i++)
	{
		switch (i)
		{
			case 0:
				mtdcr(DCRN_SDRAM0_CFGADDR, SDRAM0_B0CR);
				break;
			case 1:
				mtdcr(DCRN_SDRAM0_CFGADDR, SDRAM0_B1CR);
				break;
			case 2:
				mtdcr(DCRN_SDRAM0_CFGADDR, SDRAM0_B2CR);
				break;
			case 3:
				mtdcr(DCRN_SDRAM0_CFGADDR, SDRAM0_B3CR);
				break;
		}

		bank_config = mfdcr(DCRN_SDRAM0_CFGDATA);

		if (!(bank_config & SDRAM_CONFIG_BANK_ENABLE))
			continue;
		switch (SDRAM_CONFIG_BANK_SIZE(bank_config))
		{
			case SDRAM_CONFIG_SIZE_8M:
				mem_size += PPC440_MEM_SIZE_8M;
				break;
			case SDRAM_CONFIG_SIZE_16M:
				mem_size += PPC440_MEM_SIZE_16M;
				break;
			case SDRAM_CONFIG_SIZE_32M:
				mem_size += PPC440_MEM_SIZE_32M;
				break;
			case SDRAM_CONFIG_SIZE_64M:
				mem_size += PPC440_MEM_SIZE_64M;
				break;
			case SDRAM_CONFIG_SIZE_128M:
				mem_size += PPC440_MEM_SIZE_128M;
				break;
			case SDRAM_CONFIG_SIZE_256M:
				mem_size += PPC440_MEM_SIZE_256M;
				break;
			case SDRAM_CONFIG_SIZE_512M:
				mem_size += PPC440_MEM_SIZE_512M;
				break;
		}
	}
	return mem_size;
#endif
}

static void __init
ppc440_init_irq(void)
{
	ppc4xx_pic_init();
}

#if defined(CONFIG_BRCD_CPLD_IDE)
static void
cpld_ide_init_hwif_ports(hw_regs_t * hw,
			 unsigned long data_port, unsigned long ctrl_port,
			 int *irq)
{
	unsigned long *p;
	int i;
	static uint8_t cpld_data, *cpld_base, *ata_base;

	// TODO - Temporary hack until the CPLD capabilities bit is fix to not conflict with meteor.
	if(silkworm.board != SWBD23 && silkworm.board != SWBD29 &&
	   silkworm.board != SWBD32 && silkworm.board != SWBD34 &&
	   silkworm.board != SWBD37 && silkworm.board != SWBD43 &&
	   silkworm.board != SWBD44 && silkworm.board != SWBD45 &&
	   silkworm.board != SWBD51 && silkworm.board != SWBD58 &&
	   silkworm.board != SWBD61 && silkworm.board != SWBD70)
		return;

	p = hw->io_ports;
	*p = 0;
	if (irq)
		*irq = 0;

	printk("cpld_ide_init_hwif_ports: dp=%lu, cp=%lu, hw=%p\n", data_port, ctrl_port, hw);
	
	/* IS this correct?  How else do we register only ide0 ? */
	if(ata_base)
		return;

	if (data_port != 0)
		return;
	
	if ((cpld_base = (void*) ioremap64(PULSAR_CPLD_BASE_ADDR, PULSAR_CPLD_SIZE)) == NULL) {
		printk(KERN_WARNING "cpld_ide_init_hwif_ports: failed ioremap of cpld\n");
		return;
	}
	if ((ata_base = (void*) ioremap64(PULSAR_CPLD_IDE_BASE_ADDR, PULSAR_CPLD_IDE_SIZE)) == NULL) {
		printk(KERN_WARNING "cpld_ide_init_hwif_ports: failed ioremap of task file\n");
		return;
	}

	/* Probe for ATA/CPLD controller */
	cpld_data = readb(cpld_base + PULSAR_CPLD_REV_REG);
	printk("read cpld_data 0x%02x\n", cpld_data);

	if(cpld_data & PULSAR_REV_ATA_CAP) {
		
		/* assign the hw ports */
		//hack - the ide driver uses inb() to do the read - inb() adds _IO_BASE to the
		//address before reading - need to subtract that off here, or get the ide driver
		//to use readb instead (but still work with PCI ide controllers...)

		for (i = 0; i <= IDE_STATUS_OFFSET; i++) {
			*p++ = (unsigned long) 
				ata_base + PULSAR_REGISTER_FILE1_REG + i - _IO_BASE;
		}
		hw->io_ports[IDE_CONTROL_OFFSET] = 
			(unsigned long) ata_base + PULSAR_REGISTER_FILE2_REG - _IO_BASE;
	
		if (irq)
			*irq = 28;		//todo - make this a define

		ide_hwifs[data_port].noprobe = 0;

		printk("Silkworm CE-2 CPLD ATA interface configured [CPLD version %d]\n", 
			readb(cpld_base + PULSAR_CPLD_REV_REG) & 0x1f);
	}
	else {
		iounmap(ata_base);
		iounmap(cpld_base);
	}
}
#endif

void __init platform_init(unsigned long r3, unsigned long r4,
		unsigned long r5, unsigned long r6, unsigned long r7)
{
	bd_t *bip = (bd_t *) __res;

	parse_bootinfo(find_bootinfo());

	/*
	 * If we were passed in a board information, copy it into the
	 * residual data area.
	 */
	if (r3)
		memcpy((void *)bip, (void *) (r3 + KERNELBASE), sizeof (bd_t));

#if defined(CONFIG_PPC405_WDT)
/* Look for wdt= option on command line */
	if (strstr(cmd_line, "wdt=")) {
		int valid_wdt = 0;
		char *p, *q;
		for (q = cmd_line; (p = strstr(q, "wdt=")) != 0;) {
			q = p + 4;
			if (p > cmd_line && p[-1] != ' ')
				continue;
			wdt_period = simple_strtoul(q, &q, 0);
			valid_wdt = 1;
			++q;
		}
		wdt_enable = valid_wdt;
	}
#endif
	ppc_md.init_IRQ = ppc440_init_irq;
	ppc_md.find_end_of_memory = ppc440_find_end_of_memory;
	ppc_md.restart = ppc440_restart;
	ppc_md.power_off = ppc440_power_off;
	ppc_md.halt = ppc440_halt;

#ifdef CONFIG_SERIAL_TEXT_DEBUG
	ppc_md.progress = gen550_progress;
#endif /* CONFIG_SERIAL_TEXT_DEBUG */
#ifdef CONFIG_KGDB
	ppc_md.kgdb_map_scc = gen550_kgdb_map_scc;
#endif
	/*
	 * The Abatron BDI JTAG debugger does not tolerate others
	 * mucking with the debug registers.
	 */
#if !defined(CONFIG_BDI_SWITCH)
	/* Enable internal debug mode */
	mtspr(SPRN_DBCR0, (DBCR0_IDM));

	/* Clear any residual debug events */
	mtspr(SPRN_DBSR, 0xffffffff);
#endif

	ppc_md.setup_arch = ppc440_setup_arch;
	ppc_md.show_cpuinfo = ppc440_show_cpuinfo;

	ppc_md.get_irq = NULL;		/* Set in ppc4xx_pic_init() */
	ppc_md.calibrate_decr = ppc440_calibrate_decr;

#ifdef CONFIG_PPC405_WDT
	ppc_md.heartbeat = ppc4xx_wdt_heartbeat;
#endif
	ppc_md.heartbeat_count = 0;

#if defined(CONFIG_BRCD_CPLD_IDE)
	ppc_ide_md.ide_init_hwif = cpld_ide_init_hwif_ports;
#endif
	ppc_md.feature_call = ppc44x_do_feature_call;

	return;
}
