/*
 * arch/ppc/platforms/ebony.h
 *
 * Ebony board definitions
 *
 * Matt Porter <mporter@mvista.com>
 *
 * Copyright 2002 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 *  05/07/02 - Armin
 *  removed IIC_PORT_DFNS 
 */
#ifdef __KERNEL__
#ifndef __ASM_SILKWORM440_H__
#define __ASM_SILKWORM440_H__

#include <linux/config.h>
#include <platforms/4xx/ibm440gp.h>

#ifndef __ASSEMBLY__
typedef struct board_info {
	unsigned char	 bi_s_version[4];	/* Version of this structure */
	unsigned char	 bi_r_version[30];	/* Version of the IBM ROM */
	unsigned int	 bi_memsize;		/* DRAM installed, in bytes */
	unsigned char	 bi_enetaddr[2][6];	/* Local Ethernet MAC address */
	unsigned int	 bi_intfreq;		/* Processor speed, in Hz */
	unsigned int	 bi_busfreq;		/* PLB Bus speed, in Hz */
	unsigned int	 bi_pci_busfreq;	/* PCI Bus speed, in Hz */
} bd_t;
#define bi_tbfreq bi_intfreq

#endif /* __ASSEMBLY__ */

#define PPC440_MACHINE_NAME	"Brocade Silkworm440"

/* F/W TLB mapping used in bootloader glue to reset EMAC */
#define PPC44x_EMAC0_MR0		0xEF000800

/* Macros to get at Ebony VPD info */
#define PPC440_VPD_BASE			0x00000001fffffe00ULL
#define PPC440_VPD_SIZE			0x24
#define PPC440_NA0_OFFSET		0x0c
#define PPC440_NA1_OFFSET		0x18
#define PPC440_NA0_ADDR(base)		(base + PPC440_NA0_OFFSET)
#define PPC440_NA1_ADDR(base)		(base + PPC440_NA1_OFFSET)

/* Default clock rates for Rev. B and Rev. C silicon */
#define PPC440_440GP_RB_SYSCLK		33000000
#define PPC440_440GP_RC_SYSCLK		400000000

/* RTC/NVRAM location */
#define PPC440_RTC_ADDR			0x0000000148000000ULL
#define PPC440_RTC_SIZE			0x2000

/* Flash */
#define PPC440_FPGA_ADDR		0x0000000148300000ULL
#define PPC440_BOOT_SMALL_FLASH(x)	(x & 0x20)
#define PPC440_ONBRD_FLASH_EN(x)		(x & 0x02)
#define PPC440_FLASH_SEL(x)		(x & 0x01)
#define PPC440_SMALL_FLASH_LOW1		0x00000001ff800000ULL
#define PPC440_SMALL_FLASH_LOW2		0x00000001ff880000ULL
#define PPC440_SMALL_FLASH_HIGH1	0x00000001fff00000ULL
#define PPC440_SMALL_FLASH_HIGH2	0x00000001fff80000ULL
#define PPC440_SMALL_FLASH_SIZE		0x80000
#define PPC440_LARGE_FLASH_LOW		0x00000001ff800000ULL
#define PPC440_LARGE_FLASH_HIGH		0x00000001ffc00000ULL
#define PPC440_LARGE_FLASH_SIZE		0x400000

#define PPC440_SMALL_FLASH_BASE		0x00000001fff80000ULL
#define PPC440_LARGE_FLASH_BASE		0x00000001ff800000ULL

/*
 * Serial port defines
 */
#define RS_TABLE_SIZE	2

/* OpenBIOS defined UART mappings, used before early_serial_setup */
/* the code in head_440.S sets up similar TLB mapping for early access to serial ports */
#define UART0_IO_BASE	(u8 *) 0xEF000200
#define UART1_IO_BASE	(u8 *) 0xEF000300
#define UART0_INT	0
#define UART1_INT	1

/* Base baud is now calculated automatically in platform_init() */
#define BASE_BAUD	(4290322/16)	/* PLB freq / CPC0_CR0[UDIV] / 16 */
#define PPC44x_EXT_SERIAL_CLK_11_0592MHZ (11059200)  /* Meteor uses 11MHz clock */
#define PPC44x_EXT_SERIAL_CLK_11_1111MHZ (11111100)


#define STD_UART_OP(num)					\
	{ 0, BASE_BAUD, 0, UART##num##_INT,			\
		(ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST),	\
		iomem_base: UART##num##_IO_BASE,		\
		io_type: SERIAL_IO_MEM},

#if defined(CONFIG_UART0_TTYS0)
#define SERIAL_DEBUG_IO_BASE	UART0_IO_BASE
#define SERIAL_PORT_DFNS	\
	STD_UART_OP(0)		\
	STD_UART_OP(1)
#endif

#if defined(CONFIG_UART0_TTYS1)
#define SERIAL_DEBUG_IO_BASE	UART1_IO_BASE
#define SERIAL_PORT_DFNS	\
	STD_UART_OP(1)		\
	STD_UART_OP(0)
#endif

/* PCI support */
#define PPC440_PCI_LOWER_IO	0x00000000
#define PPC440_PCI_UPPER_IO	0x0000ffff
#define PPC440_PCI_LOWER_MEM	0x40000000
#define PPC440_PCI_UPPER_MEM	0xcfffffff

#define PPC440_PCI_CFGREGS_BASE	0x000000020ec00000
#define PPC440_PCI_CFGA_PLB32	0x000000020ec00000
#define PPC440_PCI_CFGD_PLB32	0x000000020ec00004

#define PPC440_PCI_IO_BASE	0x0000000208000000
#define PPC440_PCI_IO_SIZE	0x00010000

#define PPC440_PCI_MEM_OFFSET	0x00000000

/* CPLD addresses */
#define CPLD_BASE_ADDR		0x00000001f8000000
#define CPLD_SIZE		0x200
#define CPLD_ATA_BASE_ADDR	0x00000001f8f00000
#define CPLD_ATA_SIZE		0x3f8
#define CPLD_ATA_INTERRUPT	

/* Fixed interrupt allocation */

#endif				/* __ASM_SILKWORM440_H__ */
#endif				/* __KERNEL__ */
