/*
 * Brocade platform support for the 440EPX based products.
 * Based on: arch/ppc/platforms/4xx/sequoia.c 
 *           - Copyright 2006-2007 DENX Software Engineering, Stefan Roese <sr@denx.de>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */
#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blkdev.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/ide.h>
#include <linux/irq.h>
#include <linux/initrd.h>
#include <linux/irq.h>
#include <linux/seq_file.h>
#include <linux/root_dev.h>
#include <linux/tty.h>
#include <linux/serial.h>
#include <linux/serial_core.h>
#include <brocade/cpld.h>
#include <linux/ethtool.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/ocp.h>
#include <asm/pci-bridge.h>
#include <asm/time.h>
#include <asm/todc.h>
#include <asm/bootinfo.h>
#include <asm/ppc4xx_pic.h>
#include <asm/ppcboot.h>
#include <syslib/gen550.h>
#include <syslib/ibm440gx_common.h>

#include <brocade/platform.h>
#include <brocade/silkworm.h>
#include <brocade/silkworm_features.h>
#include "silkworm_cpld.h"

static unsigned long opb_freq, mal_freq, per_freq;

static struct ibm44x_clocks clocks __initdata;

/* Base address for the PCIX configuration registers */
int *pcix_reg_base;

extern sw_bd silkworm;
spinlock_t sdr_spinlock=SPIN_LOCK_UNLOCKED;     /* global spinlock for the sdr registers */

extern void ppc4xx_wdt_heartbeat(void);
extern int wdt_enable;
extern unsigned long wdt_period;

extern int cpld_init(void);
extern void gen550_progress(char *, unsigned short);
extern void gen550_init(int, struct uart_port *);

static void
cpld_tomahawk_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port, 
					unsigned long ctrl_port, int *irq);

static void
cpld_bulova_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port, 
					unsigned long ctrl_port, int *irq);

static void
cpld_leda_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port,
					unsigned long ctrl_port, int *irq);

static void
cpld_spike_bp_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port,
                    unsigned long ctrl_port, int *irq);

static void
cpld_spike_cp_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port,
                                        unsigned long ctrl_port, int *irq);

static void
cpld_falcon2_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port, 
					unsigned long ctrl_port, int *irq);

extern bd_t __res;

static struct ibm44x_clocks clocks __initdata;

/*
 * IRQ triggering/polarity settings for the Athena board.
 */

unsigned char *ppc4xx_uic_ext_irq_cfg __initdata = NULL;

/* 
 * flag for PCI scan, Yes=1, No=0
 */
int silkworm_pci_standalone_scan = 0;


/*
 * IRQ triggering/polarity settings for the Sequoia board.
 */
static u_char sequoia_irq_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index0 - IRQ4: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index1 - IRQ7: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index2 - IRQ8: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index3 - IRQ9: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index4 - IRQ0: Ethernet 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index5 - IRQ1: Ethernet 1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index6 - IRQ5: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index7 - IRQ6: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index8 - IRQ2: PCI slots */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index9 - IRQ3: STTM alert */
};

/*
 * IRQ triggering/polarity settings for the Tomahawk board.
 */
static u_char tomahawk_irq_initsenses[] __initdata = {

        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index0 - IRQ4: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index1 - IRQ7: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index2 - IRQ8: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index3 - IRQ9: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index4 - IRQ0: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index5 - IRQ1: asic int */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index6 - IRQ5: CF int*/
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index7 - IRQ6: PCI PERR */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index8 - IRQ2: SFP int */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index9 - IRQ3: ATT int */
};

/*
 * IRQ triggering/polarity settings for the Blazer3 (Dirk) board.
 */
static u_char blazer3_irq_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index0 - IRQ4: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index1 - IRQ7: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index2 - IRQ8: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index3 - IRQ9: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index4 - IRQ0: 440_IRQ_RSVR */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index5 - IRQ1: asic int */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index6 - IRQ5: CF int*/
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index7 - IRQ6: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index8 - IRQ2: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index9 - IRQ3: Sys */
};

/*
 * IRQ triggering/polarity settings for the Kestrel2 board.
 */
static u_char kestrel2_irq_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index0 - IRQ4: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index1 - IRQ7: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index2 - IRQ8: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index3 - IRQ9: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index4 - IRQ0: 440_IRQ_RSVR */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index5 - IRQ1: asic int */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index6 - IRQ5: CF int*/
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index7 - IRQ6: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index8 - IRQ2: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index9 - IRQ3: Sys */
};

/*
 * IRQ triggering/polarity settings for the Dragon2 board.
 */
static u_char dragon2_irq_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index0 - IRQ4: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index1 - IRQ7: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index2 - IRQ8: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index3 - IRQ9: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index4 - IRQ0: 440_IRQ_RSVR */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index5 - IRQ1: asic int */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index6 - IRQ5: CF int*/
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index7 - IRQ6: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index8 - IRQ2: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index9 - IRQ3: Sys */
};

/*
 * IRQ triggering/polarity settings for the Dragon2T board.
 */
static u_char dragon2t_irq_initsenses[] __initdata = {
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index0 - IRQ4: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index1 - IRQ7: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index2 - IRQ8: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index3 - IRQ9: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index4 - IRQ0: 440_IRQ_RSVR */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index5 - IRQ1: asic int */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index6 - IRQ5: CF int*/
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index7 - IRQ6: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index8 - IRQ2: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index9 - IRQ3: Sys */
};
/*
 * IRQ triggering/polarity settings for the Superhawk2 board.
 */
static u_char superhawk2_irq_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index0 - IRQ4: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index1 - IRQ7: CMC FRU */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index2 - IRQ8: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index3 - IRQ9: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index4 - IRQ0: 440_IRQ_RSVR */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index5 - IRQ1: asic int */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index6 - IRQ5: CF int*/
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index7 - IRQ6: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index8 - IRQ2: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index9 - IRQ3: Sys */
};

/*
 * IRQ triggering/polarity settings for the Bulova board.
 */
static u_char bulova_irq_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 27: Ext Int 4 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 50: Ext Int 7 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 51: Ext Int 8 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 52: Ext Int 9 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 60: Ext Int 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 62: Ext Int 1 - GoldenEye */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 64: Ext Int 5 - Compact Flash */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 65: Ext Int 6 - Mictor connector */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 67: Ext Int 2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 68: Ext Int 3 - Sys Ints */
};

/*
 * IRQ triggering/polarity settings for the Leda board.
 */
static u_char leda_irq_initsenses[] __initdata = {

	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index0 - IRQ4: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index1 - IRQ7: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index2 - IRQ8: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index3 - IRQ9: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index4 - IRQ0: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index5 - IRQ1: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index6 - IRQ5: CF int*/
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index7 - IRQ6: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index8 - IRQ2: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index9 - IRQ3: PCIe/PCI/GE-Phy*/
};

/*
 * IRQ triggering/polarity settings for the Europa board.
 */
static u_char europa_irq_initsenses[] __initdata = {

	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index0 - IRQ4: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index1 - IRQ7: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index2 - IRQ8: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index3 - IRQ9: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index4 - IRQ0: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index5 - IRQ1: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index6 - IRQ5: CF int*/
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index7 - IRQ6: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index8 - IRQ2: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index9 - IRQ3: PCIe/PCI/GE-Phy*/
};

/*
 * IRQ triggering/polarity settings for the Spike-BP board.
 */
static u_char spike_bp_irq_initsenses[] __initdata = {

    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index0 - IRQ4: */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index1 - IRQ7: */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index2 - IRQ8: */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index3 - IRQ9: */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index4 - IRQ0: */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index5 - IRQ1: */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index6 - IRQ5: CF int*/
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index7 - IRQ6: */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index8 - IRQ2: */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index9 - IRQ3: */
};


/*
 * IRQ triggering/polarity settings for the Spike-CP board.
 */
static u_char spike_cp_irq_initsenses[] __initdata = {

        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index0 - IRQ4: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index1 - IRQ7: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index2 - IRQ8: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index3 - IRQ9: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index4 - IRQ0: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index5 - IRQ1: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Index6 - IRQ5: CF int*/
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index7 - IRQ6: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index8 - IRQ2: */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Index9 - IRQ3: PCIe/PCI/GE-Phy*/
};

/*
 * IRQ triggering/polarity settings for the Falcon2 board.
 */
static u_char falcon2_irq_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 27: Ext Int 4 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 50: Ext Int 7 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 51: Ext Int 8 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 52: Ext Int 9 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 60: Ext Int 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 62: Ext Int 1 - GoldenEye */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 64: Ext Int 5 - Compact Flash */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 65: Ext Int 6 - Mictor connector */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 67: Ext Int 2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 68: Ext Int 3 - Sys Ints */
};

/*
 * IRQ triggering/polarity settings for Pharos
 */
static u_char pharos_irq_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 0 - IRQ4: PHY 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 1 - IRQ7: PHY 1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 2 - IRQ8: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 3 - IRQ9: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 4 - IRQ0: CME */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 5 - IRQ1: ASIC */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 6 - IRQ5: CF */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 7 - IRQ6: PCI ERR */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 8 - IRQ2: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 9 - IRQ3: system */
};

/*
 * IRQ triggering/polarity settings for Bulova2
 */
static u_char bulova2_irq_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 0 - IRQ4: PHY 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 1 - IRQ7: PHY 1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 2 - IRQ8: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 3 - IRQ9: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 4 - IRQ0: CME */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 5 - IRQ1: ASIC */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 6 - IRQ5: CF */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 7 - IRQ6: PCI ERR */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 8 - IRQ2: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 9 - IRQ3: system */
};
/*
 * IRQ triggering/polarity settings for Kestrel3
 */
static u_char kestrel3_irq_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 0 - IRQ4: reserve,pull low*/
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 1 - IRQ7: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 2 - IRQ8: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 3 - IRQ9: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 4 - IRQ0: 440 GPIO INT */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 5 - IRQ1: ASIC */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 6 - IRQ5: CF */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 7 - IRQ6: PCI ERR */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 8 - IRQ2: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 9 - IRQ3: system */
};


/*
 * IRQ triggering/polarity settings for Superhawk3
 */
static u_char superhawk3_irq_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 0 - IRQ4: unused high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 1 - IRQ7: CMC_REREAD_FRU */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 2 - IRQ8: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 3 - IRQ9: unused low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 4 - IRQ0: unused high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 5 - IRQ1: ASIC */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 6 - IRQ5: CF */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 7 - IRQ6: PCI ERR */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 8 - IRQ2: unused high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 9 - IRQ3: system */
};

/*
 * IRQ triggering/polarity settings for Falcon3
 */
static u_char falcon3_irq_initsenses[] __initdata = {
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 0 - IRQ4: Reverse IRQ1: low  */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 1 - IRQ7: Unused - pulled low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 2 - IRQ8: Unused - pulled low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 3 - IRQ9: Unused - pulled low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 4 - IRQ0: 440 GPIO Interrupt: low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 5 - IRQ1: Condor3 via System FPGA: high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 6 - IRQ5: Compact Flash card :high */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* 7 - IRQ6: PCI parity error :low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 8 - IRQ2: Unused - pulled low */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* 9 - IRQ3: Serial interface state: high */
};


/*
 * NOR FLASH configuration (using mtd physmap driver)
 */

/* start will be added dynamically, end is always fixed */
#if 0
static struct resource sequoia_nor_resource = {
		.end   = 0xffffffff,
		.flags = IORESOURCE_MEM,
};

#define RW_PART0_OF	0
#define RW_PART0_SZ	0x180000
#define RW_PART1_SZ	0x280000
/* Partition 2 will be autosized dynamically... */
#define RW_PART3_SZ	0x40000
#define RW_PART4_SZ	0x60000

static struct mtd_partition sequoia_nor_parts[] = {
	{
		.name = "kernel",
		.offset = 0,
		.size = RW_PART0_SZ
	},
	{
		.name = "root",
		.offset = MTDPART_OFS_APPEND,
		.size = RW_PART1_SZ,
	},
	{
		.name = "user",
		.offset = MTDPART_OFS_APPEND,
/*		.size = RW_PART2_SZ */ /* will be adjusted dynamically */
	},
	{
		.name = "env",
		.offset = MTDPART_OFS_APPEND,
		.size = RW_PART3_SZ,
	},
	{
		.name = "u-boot",
		.offset = MTDPART_OFS_APPEND,
		.size = RW_PART4_SZ,
	}
};

static struct physmap_flash_data sequoia_nor_data = {
	.width		= 2,
	.parts		= sequoia_nor_parts,
	.nr_parts	= ARRAY_SIZE(sequoia_nor_parts),
};

static struct platform_device sequoia_nor_device = {
	.name		= "physmap-flash",
	.id		= 0,
	.dev = {
			.platform_data = &sequoia_nor_data,
		},
	.num_resources	= 1,
	.resource	= &sequoia_nor_resource,
};

/*
 * NAND FLASH configuration (using 440EP(x) NDFC)
 */
static struct resource sequoia_ndfc = {
	.start = (u32)SEQUOIA_NAND_FLASH_REG_ADDR,
	.end = (u32)SEQUOIA_NAND_FLASH_REG_ADDR + SEQUOIA_NAND_FLASH_REG_SIZE,
	.flags = IORESOURCE_MEM,
};

/* todo: add logic to detect booting from NAND (NAND on CS0) */
#define CS_NAND_0	3	/* use chip select 3 for NAND device 0 */

static struct mtd_partition sequoia_nand_parts[] = {
        {
                .name   = "content",
                .offset = 0,
                .size   = MTDPART_SIZ_FULL,
        }
};

struct ndfc_controller_settings sequoia_ndfc_settings = {
	.ccr_settings = (NDFC_CCR_BS(CS_NAND_0) |
			 NDFC_CCR_ARAC1),
	.ndfc_erpn = (SEQUOIA_NAND_FLASH_REG_ADDR) >> 32,
};

struct platform_nand_ctrl sequoia_nand_ctrl = {
	.priv = &sequoia_ndfc_settings,
};

static struct platform_device sequoia_ndfc_device = {
	.name = "ndfc-nand",
	.id = 0,
	.dev = {
		.platform_data = &sequoia_nand_ctrl,
	},
	.num_resources = 1,
	.resource = &sequoia_ndfc,
};

static struct ndfc_chip_settings sequoia_chip0_settings = {
	.bank_settings = 0x80002222,
};

static struct nand_ecclayout nand_oob_16 = {
	.eccbytes = 6,
	.eccpos = {9, 10, 11, 13, 14, 15},
	.oobfree = {
		 {.offset = 8,
		  . length = 8}}
};

static struct platform_nand_chip sequoia_nand_chip0 = {
	.nr_chips = 1,
	.chip_offset = CS_NAND_0,
	.nr_partitions = ARRAY_SIZE(sequoia_nand_parts),
	.partitions = sequoia_nand_parts,
	.chip_delay = 50,
	.ecclayout = &nand_oob_16,
	.priv = &sequoia_chip0_settings,
};

static struct platform_device sequoia_nand_device = {
	.name = "ndfc-chip",
	.id = 0,
	.num_resources = 1,
	.resource = &sequoia_ndfc,
	.dev = {
		.platform_data = &sequoia_nand_chip0,
		.parent = &sequoia_ndfc_device.dev,
	}
};

static int sequoia_setup_flash(void)
{
	sequoia_nor_resource.start = __res.bi_flashstart;

	/*
	 * Adjust partition 2 to flash size
	 */
	sequoia_nor_parts[2].size = __res.bi_flashsize -
		RW_PART0_SZ - RW_PART1_SZ - RW_PART3_SZ - RW_PART4_SZ;

	platform_device_register(&sequoia_nor_device);

	/* todo: add logic to detect booting from NAND (NAND on CS0) */

	platform_device_register(&sequoia_ndfc_device);
	platform_device_register(&sequoia_nand_device);

	return 0;
}
arch_initcall(sequoia_setup_flash);
#endif

/*
 * get size of system memory from Board Info .
 */
unsigned long __init 
ppc440epx_find_end_of_memory(void)
{
	/* board info structure defined in /include/asm-ppc/ppcboot.h */
	return  __res.bi_memsize;
}

static void __init 
ppc440epx_calibrate_decr(void)
{
	unsigned int freq;

	if (mfspr(SPRN_CCR1) & CCR1_TCS)
		freq = PPC440EPX_TMRCLK;
	else
		freq = clocks.cpu;

	ibm44x_calibrate_decr(freq);

}

/*
 *   This routine pretty-prints the platform's internal CPU clock
 *   frequencies into the buffer for usage in /proc/cpuinfo.
 */

static int
ppc440epx_show_cpuinfo(struct seq_file *m)
{
	seq_printf(m, "machine\t\t: %s \n", PPC440_MACHINE_NAME);

	return 0;
}

static inline int
ppc440epx_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
#ifdef CONFIG_SILKWORM
        /* Boot PROM configures the IRQ for each PCI device.  Linux scans the device and
         * just uses what the bootrom set.
         */
        int irq;

        if(dev)
                irq = dev->irq;
        else
                irq = -1;
        return irq;
#else
	static char pci_irq_table[][4] =
	/*
	 *	PCI IDSEL/INTPIN->INTLINE
	 * 	   A   B   C   D
	 */
	{
		{ 67, 67, 67, 67 },	/* IDSEL 10 - PCI Slot 1, ext. IRQ 2 */
		{ 67, 67, 67, 67 },	/* IDSEL 11 - PCI Slot x, ext. IRQ 2 */
		{ 67, 67, 67, 67 },	/* IDSEL 12 - PCI Slot 0, ext. IRQ 2 */
	};

	const long min_idsel = 10, max_idsel = 12, irqs_per_slot = 4;
	return PCI_IRQ_TABLE_LOOKUP;
#endif
}

#define PHY_ADDR_TO_IBM_EMAC_PHY_MAP(phy_addr) (~(1 << (phy_addr)))

static void __init silkworm440epx_set_emacdata(void)
{
	struct ocp_def *def;
	struct ocp_def *tmp;
	struct ocp_func_emac_data *emacdata;
	bd_t *bip = (bd_t *) &(__res);

	u32 exc1, exc2, exc3, exc4;     /* Exclude modes for phys */
	u32 phymap1, phymap2, phymap3, phymap4;

	/* initialized to exclude nothing */
	exc1 = exc2 = exc3 = exc4 = 0;
	phymap1 =PHY_ADDR_TO_IBM_EMAC_PHY_MAP(1);
	phymap2 =PHY_ADDR_TO_IBM_EMAC_PHY_MAP(2);
	phymap3 =PHY_ADDR_TO_IBM_EMAC_PHY_MAP(3);
	phymap4 =PHY_ADDR_TO_IBM_EMAC_PHY_MAP(4);


	switch (silkworm.board) {
		/*
		 * the below PHY configurations are believed to work with
		 * these board IDs.
		 */
		case SWBD71:
		case SWBD72:
		case SWBD82:
		case SWBD66:
		case SWBD73:
		case SWBD75:
		case SWBD92:
		case SWBD86:
		case SWBD87:			
		case SWBD109:
		case SWBD118:
		case SWBD129:
		case SWBD130:
		case SWBD124:
		case SWBD125:
		case SWBD134:
		case SWBD149:
		case SWBD150:
			def = ocp_get_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 0);
			emacdata = def->additions;
			memcpy(emacdata->mac_addr, __res.bi_enetaddr, 6);
			emacdata->phy_map = phymap1;

			if(silkworm.board == SWBD134) {
				emacdata->phy_mode = PHY_MODE_RGMII;
				emacdata->phy_map = PHY_ADDR_TO_IBM_EMAC_PHY_MAP(0x10);
			}
			else if ((silkworm.board == SWBD71) || (silkworm.board == SWBD72) ||
				(silkworm.board == SWBD82) || (silkworm.board == SWBD66) || 
				(silkworm.board == SWBD73) || (silkworm.board == SWBD75) ||
				(silkworm.board == SWBD86) || (silkworm.board == SWBD87) ||
				(silkworm.board == SWBD109) || (silkworm.board == SWBD118) ||
				(silkworm.board == SWBD124) || (silkworm.board == SWBD125) ||
				(silkworm.board == SWBD129) || (silkworm.board == SWBD130)||
				(silkworm.board == SWBD149) || (silkworm.board == SWBD150))
				emacdata->phy_mode = PHY_MODE_MII;
			else
				emacdata->phy_mode = PHY_MODE_GMII;

			ocp_remove_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 1);
			break;

		case SWBD85:

			/* Set mac_addr, phy mode for each EMAC */
			def = ocp_get_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 0);
			emacdata = def->additions;
			memcpy(emacdata->mac_addr, __res.bi_enetaddr, 6);
			emacdata->phy_map = 0xffffffff;  /* PHY-LESS */
			emacdata->phy_mode = PHY_MODE_GMII;

			/* this is needed since we still have P1 boards in development lab -mlei */
			if (bip->bi_ethspeed == 1000) {
				printk("This is spike P2 board\n");
				emacdata->phy_feat_exc = SUPPORTED_Autoneg | SUPPORTED_10baseT_Full
					| SUPPORTED_10baseT_Half | SUPPORTED_100baseT_Half
					| SUPPORTED_100baseT_Full | SUPPORTED_1000baseT_Half;
			} else {
				printk("This is spike P1 board\n");
				emacdata->phy_feat_exc = SUPPORTED_Autoneg | SUPPORTED_10baseT_Full
					| SUPPORTED_10baseT_Half | SUPPORTED_100baseT_Half
					| SUPPORTED_1000baseT_Full | SUPPORTED_1000baseT_Half;
			}

			ocp_remove_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 1);
			break;

		case SWBD83: /* Spike CP*/
             /* Set mac_addr, phy mode for each EMAC */
             tmp = ocp_get_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 0);
             emacdata = tmp->additions;
             memcpy(emacdata->mac_addr, __res.bi_enetaddr, 6);
             emacdata->mac_addr[5] += 1;
             emacdata->phy_map = phymap1;
             emacdata->phy_mode = PHY_MODE_RGMII;

             def = ocp_get_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 1);
             emacdata = def->additions;
             memcpy(emacdata->mac_addr, __res.bi_enetaddr, 6);
             emacdata->phy_map = phymap2;
             emacdata->phy_mode = PHY_MODE_RGMII;

             /* reorder EMACs in OCP to swap logical interface names */
             ocp_remove_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 0);
             ocp_remove_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 1);
             ocp_add_one_device(def); 
             ocp_add_one_device(tmp);
             break;

		case SWBD78:
		case SWBD88:
			/* Set mac_addr, phy mode for each EMAC */
			tmp = ocp_get_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 0);
			emacdata = tmp->additions;
			memcpy(emacdata->mac_addr, __res.bi_enetaddr, 6);
			emacdata->mac_addr[5] += 1;
			emacdata->phy_map = phymap1;
			emacdata->phy_mode = PHY_MODE_RGMII;

			def = ocp_get_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 1);
			emacdata = def->additions;
			memcpy(emacdata->mac_addr, __res.bi_enetaddr, 6);
			emacdata->phy_map = phymap2;
			emacdata->phy_mode = PHY_MODE_RGMII;

			/* reorder EMACs in OCP to swap logical interface names */
			ocp_remove_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 0);
			ocp_remove_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 1);
			ocp_add_one_device(def);
			ocp_add_one_device(tmp);
			break;

		case SWBD117:
			/* Set mac_addr, phy mode for each EMAC */
			tmp = ocp_get_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 0);
			emacdata = tmp->additions;
			memcpy(emacdata->mac_addr, __res.bi_enetaddr, 6);
			emacdata->mac_addr[5] += 1;
			emacdata->phy_map = phymap1;
			emacdata->phy_mode = PHY_MODE_RGMII;
			emacdata->phy_feat_exc = SUPPORTED_Autoneg;

			def = ocp_get_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 1);
			emacdata = def->additions;
			memcpy(emacdata->mac_addr, __res.bi_enetaddr, 6);
			emacdata->phy_map = phymap2;
			emacdata->phy_mode = PHY_MODE_RGMII;

			/* reorder EMACs in OCP to swap logical interface names */
			ocp_remove_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 0);
			ocp_remove_one_device(OCP_VENDOR_IBM, OCP_FUNC_EMAC, 1);
			ocp_add_one_device(def);
			ocp_add_one_device(tmp);
			break;

		/* for others, you take your chances, but you've been warned */
		default:
			printk(KERN_WARNING
					"silkworm440gx_set_emac_data(): unknown SWBD %d\n",
					silkworm.board);
			break;
	}


    /*
    * Clear PLB4A0_ACR[WRP] (Write Pipeline Control)
    * This fix will make the MAL burst disabling patch for the Linux
    * EMAC driver obsolete.
    */
	mtdcr(DCRN_PLB4A0_ACR, mfdcr(DCRN_PLB4A0_ACR) & ~(0x80000000 >> 7));
}

static int ppc440epx_exclude_device(unsigned char bus, unsigned char devfn)
{
	return (bus == 0 && devfn == 0);
}

#define PCI_READW(offset) \
        (readw((void *)((u32)pci_reg_base + offset)))

#define PCI_WRITEW(value, offset) \
	(writew(value, (void *)((u32)pci_reg_base + offset)))

#define PCI_WRITEL(value, offset) \
	(writel(value, (void *)((u32)pci_reg_base + offset)))

#define PCI_CFG_OUT(offset, value) \
	(out_le32 (pci_cfg_base + offset, value))

#define PCI_CFG_IN(offset) \
	(in_le32(pci_cfg_base + offset))

static void __init tomahawk_setup_pci(void)
{
	void *pci_reg_base;
	void *pci_cfg_base;
	unsigned long memory_size;

	memory_size = ppc_md.find_end_of_memory();

	pci_reg_base = ioremap64(PPC440_PCIL0_BASE, PPC440_PCIL0_SIZE);
	pci_cfg_base = ioremap64(PPC440_PCI_CFGREGS_BASE, 64);

	PCI_CFG_OUT(PPC440_PCI_CFGA_OFFSET, 0x80000000 | (PCI_COMMAND & 0xfc));
	PCI_CFG_OUT(PPC440_PCI_CFGD_OFFSET,
		    (PCI_CFG_IN(PPC440_PCI_CFGD_OFFSET) |
		     PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER));

	/* Disable region first */
	PCI_WRITEL(0, PPC440_PCIL0_PMM0MA);

	/* PLB starting addr: 0x0000000180000000 */
	PCI_WRITEL(PPC440_PCI_PHY_MEM_BASE, PPC440_PCIL0_PMM0LA);

	/* PCI start addr, 0x80000000 (PCI Address) */
	PCI_WRITEL(PPC440_PCI_MEM_BASE, PPC440_PCIL0_PMM0PCILA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM0PCIHA);

	/* Enable no pre-fetch, enable region */
	PCI_WRITEL(((0xffffffff -
		     (PPC440_PCI_UPPER_MEM - PPC440_PCI_MEM_BASE)) | 0x01),
		   PPC440_PCIL0_PMM0MA);

	/* Disable region one */
	PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1LA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1PCILA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1PCIHA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);

	/* Disable region two */
	PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2LA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2PCILA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2PCIHA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);

	/* Now configure the PCI->PLB windows, we only use PTM1
	 *
	 * For Inbound flow, set the window size to all available memory
	 * This is required because if size is smaller,
	 * then Eth/PCI DD would fail as PCI card not able to access
	 * the memory allocated by DD.
	 */

	PCI_WRITEL(0, PPC440_PCIL0_PTM1MS);	/* disabled region 1 */
	PCI_WRITEL(0, PPC440_PCIL0_PTM1LA);	/* begin of address map */

	memory_size = 1 << fls(memory_size - 1);

	/* Size low + Enabled */
	PCI_WRITEL((0xffffffff - (memory_size - 1)) | 0x1, PPC440_PCIL0_PTM1MS);

	eieio();
	iounmap(pci_reg_base);
	iounmap(pci_cfg_base);
}

static void __init tomahawk_setup_hose(void)
{
	unsigned int bar_response, bar;
	struct pci_controller *hose;

	tomahawk_setup_pci();

	hose = pcibios_alloc_controller();

	if (!hose)
		return;

	hose->first_busno = 0;
	hose->last_busno = 0xff;

	hose->pci_mem_offset = PPC440_PCI_MEM_OFFSET;

	pci_init_resource(&hose->io_resource,
			  PPC440_PCI_LOWER_IO,
			  PPC440_PCI_UPPER_IO,
			  IORESOURCE_IO,
			  "PCI host bridge");

	pci_init_resource(&hose->mem_resources[0],
			  PPC440_PCI_LOWER_MEM,
			  PPC440_PCI_UPPER_MEM,
			  IORESOURCE_MEM,
			  "PCI host bridge");

	ppc_md.pci_exclude_device = ppc440epx_exclude_device;

	hose->io_space.start = PPC440_PCI_LOWER_IO;
	hose->io_space.end = PPC440_PCI_UPPER_IO;
	hose->mem_space.start = PPC440_PCI_LOWER_MEM;
	hose->mem_space.end = PPC440_PCI_UPPER_MEM;
	isa_io_base =
		(unsigned long)ioremap64(PPC440_PCI_IO_BASE, PPC440_PCI_IO_SIZE);
	hose->io_base_virt = (void *)isa_io_base;

	setup_indirect_pci(hose, PPC440_PCI_CFGA_PLB32, PPC440_PCI_CFGD_PLB32);
	hose->set_cfg_type = 1;

	/* Zero config bars */
	for (bar = PCI_BASE_ADDRESS_1; bar <= PCI_BASE_ADDRESS_2; bar += 4) {
		early_write_config_dword(hose, hose->first_busno,
					 PCI_FUNC(hose->first_busno), bar,
					 0x00000000);
		early_read_config_dword(hose, hose->first_busno,
					PCI_FUNC(hose->first_busno), bar,
					&bar_response);
	}

	hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

	ppc_md.pci_swizzle = common_swizzle;
	ppc_md.pci_map_irq = ppc440epx_map_irq;
}

static                  
void __init spike_cp_setup_pci(void)
{                                       
        void *pci_reg_base; 
        void *pci_cfg_base;
        unsigned long memory_size;
                        
        //memory_size = ppc_md.find_end_of_memory();
        memory_size =ppc440epx_find_end_of_memory();
        printk("spike_cp_setup_pci: memory size %lu \n",memory_size);
        
        pci_reg_base = ioremap64(PPC440_PCIL0_BASE, PPC440_PCIL0_SIZE);
        pci_cfg_base = ioremap64(PPC440_PCI_CFGREGS_BASE, 64);
        
    PCI_CFG_OUT(PPC440_PCI_CFGA_OFFSET, 0x80000000 | (PCI_COMMAND & 0xfc));
    PCI_CFG_OUT(PPC440_PCI_CFGD_OFFSET, 
            (PCI_CFG_IN(PPC440_PCI_CFGD_OFFSET) |
             PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER));
        
    /* Disable region first */          
    PCI_WRITEL(0, PPC440_PCIL0_PMM0MA);
        
    /* PLB starting addr: 0x0000000180000000 */
    PCI_WRITEL(PPC440_PCI_PHY_MEM_BASE, PPC440_PCIL0_PMM0LA);
    /* PCI start addr, 0x80000000 (PCI Address) */
    PCI_WRITEL(PPC440_PCI_MEM_BASE, PPC440_PCIL0_PMM0PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM0PCIHA);

    /* Enable no pre-fetch, enable region */
    PCI_WRITEL(((0xffffffff -
             (PPC440_PCI_UPPER_MEM - PPC440_PCI_MEM_BASE)) | 0x01),
           PPC440_PCIL0_PMM0MA);

    /* Disable region one */
    PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1LA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1PCIHA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);

    /* Disable region two */
    PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2LA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2PCIHA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);

    /* Now configure the PCI->PLB windows, we only use PTM1
     *
     * For Inbound flow, set the window size to all available memory
     * This is required because if size is smaller,
     * then Eth/PCI DD would fail as PCI card not able to access
     * the memory allocated by DD.
     */

    PCI_WRITEL(0, PPC440_PCIL0_PTM1MS); /* disabled region 1 */
    PCI_WRITEL(0, PPC440_PCIL0_PTM1LA); /* begin of address map */

    memory_size = 1 << fls(memory_size - 1);

    /* Size low + Enabled */
    PCI_WRITEL((0xffffffff - (memory_size - 1)) | 0x1, PPC440_PCIL0_PTM1MS);

    eieio();
    iounmap(pci_reg_base);
    iounmap(pci_cfg_base);
}

static
void __init spike_cp_setup_hose(void)
{
    unsigned int bar_response, bar;
    struct pci_controller *hose;


    spike_cp_setup_pci();

    hose = pcibios_alloc_controller();

    if (!hose)
        return;

    hose->first_busno = 0;
    hose->last_busno = 0xff;

    hose->pci_mem_offset = PPC440_PCI_MEM_OFFSET;

    pci_init_resource(&hose->io_resource,
              PPC440_PCI_LOWER_IO,
              PPC440_PCI_UPPER_IO,
              IORESOURCE_IO,
              "PCI host bridge");
    pci_init_resource(&hose->mem_resources[0],
              PPC440_PCI_LOWER_MEM,
              PPC440_PCI_UPPER_MEM,
              IORESOURCE_MEM,
              "PCI host bridge");
    ppc_md.pci_exclude_device = ppc440epx_exclude_device;

    hose->io_space.start = PPC440_PCI_LOWER_IO;
    hose->io_space.end = PPC440_PCI_UPPER_IO;
    hose->mem_space.start = PPC440_PCI_LOWER_MEM;
    hose->mem_space.end = PPC440_PCI_UPPER_MEM;
    isa_io_base =
        (unsigned long)ioremap64(PPC440_PCI_IO_BASE, PPC440_PCI_IO_SIZE);
    hose->io_base_virt = (void *)isa_io_base;

    setup_indirect_pci(hose, PPC440_PCI_CFGA_PLB32, PPC440_PCI_CFGD_PLB32);
    hose->set_cfg_type = 1;

    /* Zero config bars */
    for (bar = PCI_BASE_ADDRESS_1; bar <= PCI_BASE_ADDRESS_2; bar += 4) {
        early_write_config_dword(hose, hose->first_busno,
                     PCI_FUNC(hose->first_busno), bar,
                     0x00000000);
        early_read_config_dword(hose, hose->first_busno,
                    PCI_FUNC(hose->first_busno), bar,
                    &bar_response);
    }
    hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

    ppc_md.pci_swizzle = common_swizzle;
    ppc_md.pci_map_irq = ppc440epx_map_irq;
}

static void __init bulova_setup_pci(void)
{
	void *pci_reg_base;
	void *pci_cfg_base;
	unsigned long memory_size;

	memory_size = ppc_md.find_end_of_memory();

	pci_reg_base = ioremap64(PPC440_PCIL0_BASE, PPC440_PCIL0_SIZE);
	pci_cfg_base = ioremap64(PPC440_PCI_CFGREGS_BASE, 64);

	PCI_CFG_OUT(PPC440_PCI_CFGA_OFFSET, 0x80000000 | (PCI_COMMAND & 0xfc));
	PCI_CFG_OUT(PPC440_PCI_CFGD_OFFSET,
		    (PCI_CFG_IN(PPC440_PCI_CFGD_OFFSET) |
		     PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER));

	/* Disable region first */
	PCI_WRITEL(0, PPC440_PCIL0_PMM0MA);

	/* PLB starting addr: 0x0000000180000000 */
	PCI_WRITEL(PPC440_PCI_PHY_MEM_BASE, PPC440_PCIL0_PMM0LA);

	/* PCI start addr, 0x80000000 (PCI Address) */
	PCI_WRITEL(PPC440_PCI_MEM_BASE, PPC440_PCIL0_PMM0PCILA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM0PCIHA);

	/* Enable no pre-fetch, enable region */
	PCI_WRITEL(((0xffffffff -
		     (PPC440_PCI_UPPER_MEM - PPC440_PCI_MEM_BASE)) | 0x01),
		   PPC440_PCIL0_PMM0MA);

	/* Disable region one */
	PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1LA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1PCILA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1PCIHA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);

	/* Disable region two */
	PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2LA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2PCILA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2PCIHA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);

	/* Now configure the PCI->PLB windows, we only use PTM1
	 *
	 * For Inbound flow, set the window size to all available memory
	 * This is required because if size is smaller,
	 * then Eth/PCI DD would fail as PCI card not able to access
	 * the memory allocated by DD.
	 */

	PCI_WRITEL(0, PPC440_PCIL0_PTM1MS);	/* disabled region 1 */
	PCI_WRITEL(0, PPC440_PCIL0_PTM1LA);	/* begin of address map */

	memory_size = 1 << fls(memory_size - 1);

	/* Size low + Enabled */
	PCI_WRITEL((0xffffffff - (memory_size - 1)) | 0x1, PPC440_PCIL0_PTM1MS);

	eieio();
	iounmap(pci_reg_base);
	iounmap(pci_cfg_base);
}

static void __init bulova_setup_hose(void)
{
	unsigned int bar_response, bar;
	struct pci_controller *hose;

	bulova_setup_pci();

	hose = pcibios_alloc_controller();

	if (!hose)
		return;

	hose->first_busno = 0;
	hose->last_busno = 0xff;

	hose->pci_mem_offset = PPC440_PCI_MEM_OFFSET;

	pci_init_resource(&hose->io_resource,
			  PPC440_PCI_LOWER_IO,
			  PPC440_PCI_UPPER_IO,
			  IORESOURCE_IO,
			  "PCI host bridge");

	pci_init_resource(&hose->mem_resources[0],
			  PPC440_PCI_LOWER_MEM,
			  PPC440_PCI_UPPER_MEM,
			  IORESOURCE_MEM,
			  "PCI host bridge");

	ppc_md.pci_exclude_device = ppc440epx_exclude_device;

	hose->io_space.start = PPC440_PCI_LOWER_IO;
	hose->io_space.end = PPC440_PCI_UPPER_IO;
	hose->mem_space.start = PPC440_PCI_LOWER_MEM;
	hose->mem_space.end = PPC440_PCI_UPPER_MEM;
	isa_io_base =
		(unsigned long)ioremap64(PPC440_PCI_IO_BASE, PPC440_PCI_IO_SIZE);
	hose->io_base_virt = (void *)isa_io_base;

	setup_indirect_pci(hose, PPC440_PCI_CFGA_PLB32, PPC440_PCI_CFGD_PLB32);
	hose->set_cfg_type = 1;

	/* Zero config bars */
	for (bar = PCI_BASE_ADDRESS_1; bar <= PCI_BASE_ADDRESS_2; bar += 4) {
		early_write_config_dword(hose, hose->first_busno,
					 PCI_FUNC(hose->first_busno), bar,
					 0x00000000);
		early_read_config_dword(hose, hose->first_busno,
					PCI_FUNC(hose->first_busno), bar,
					&bar_response);
	}

	hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

	ppc_md.pci_swizzle = common_swizzle;
	ppc_md.pci_map_irq = ppc440epx_map_irq;
}

static
void __init leda_setup_pci(void)
{
	void *pci_reg_base;
	void *pci_cfg_base;
	unsigned long memory_size;

	memory_size = ppc_md.find_end_of_memory();

	pci_reg_base = ioremap64(PPC440_PCIL0_BASE, PPC440_PCIL0_SIZE);
	pci_cfg_base = ioremap64(PPC440_PCI_CFGREGS_BASE, 64);

    PCI_CFG_OUT(PPC440_PCI_CFGA_OFFSET, 0x80000000 | (PCI_COMMAND & 0xfc));
    PCI_CFG_OUT(PPC440_PCI_CFGD_OFFSET,
            (PCI_CFG_IN(PPC440_PCI_CFGD_OFFSET) |
             PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER));

    /* Disable region first */
    PCI_WRITEL(0, PPC440_PCIL0_PMM0MA);

    /* PLB starting addr: 0x0000000180000000 */
    PCI_WRITEL(PPC440_PCI_PHY_MEM_BASE, PPC440_PCIL0_PMM0LA);
    /* PCI start addr, 0x80000000 (PCI Address) */
    PCI_WRITEL(PPC440_PCI_MEM_BASE, PPC440_PCIL0_PMM0PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM0PCIHA);

    /* Enable no pre-fetch, enable region */
    PCI_WRITEL(((0xffffffff -
             (PPC440_PCI_UPPER_MEM - PPC440_PCI_MEM_BASE)) | 0x01),
           PPC440_PCIL0_PMM0MA);

    /* Disable region one */
    PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1LA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1PCIHA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);

    /* Disable region two */
    PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2LA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2PCIHA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);

    /* Now configure the PCI->PLB windows, we only use PTM1
     *
     * For Inbound flow, set the window size to all available memory
     * This is required because if size is smaller,
     * then Eth/PCI DD would fail as PCI card not able to access
     * the memory allocated by DD.
     */

    PCI_WRITEL(0, PPC440_PCIL0_PTM1MS); /* disabled region 1 */
    PCI_WRITEL(0, PPC440_PCIL0_PTM1LA); /* begin of address map */

    memory_size = 1 << fls(memory_size - 1);

    /* Size low + Enabled */
    PCI_WRITEL((0xffffffff - (memory_size - 1)) | 0x1, PPC440_PCIL0_PTM1MS);

    eieio();
    iounmap(pci_reg_base);
    iounmap(pci_cfg_base);
}

static
void __init leda_setup_hose(void)
{
    unsigned int bar_response, bar;
    struct pci_controller *hose;

	/* disabled for Leda */
	printk(KERN_INFO "Silkworm: Suppressing PCI init code for Leda SWBD78\n");
	return;

    leda_setup_pci();

    hose = pcibios_alloc_controller();

    if (!hose)
        return;

    hose->first_busno = 0;
    hose->last_busno = 0xff;

    hose->pci_mem_offset = PPC440_PCI_MEM_OFFSET;

    pci_init_resource(&hose->io_resource,
              PPC440_PCI_LOWER_IO,
              PPC440_PCI_UPPER_IO,
              IORESOURCE_IO,
              "PCI host bridge");
    pci_init_resource(&hose->mem_resources[0],
              PPC440_PCI_LOWER_MEM,
              PPC440_PCI_UPPER_MEM,
              IORESOURCE_MEM,
              "PCI host bridge");

    ppc_md.pci_exclude_device = ppc440epx_exclude_device;

    hose->io_space.start = PPC440_PCI_LOWER_IO;
    hose->io_space.end = PPC440_PCI_UPPER_IO;
    hose->mem_space.start = PPC440_PCI_LOWER_MEM;
    hose->mem_space.end = PPC440_PCI_UPPER_MEM;
    isa_io_base =
        (unsigned long)ioremap64(PPC440_PCI_IO_BASE, PPC440_PCI_IO_SIZE);
    hose->io_base_virt = (void *)isa_io_base;

    setup_indirect_pci(hose, PPC440_PCI_CFGA_PLB32, PPC440_PCI_CFGD_PLB32);
    hose->set_cfg_type = 1;

    /* Zero config bars */
    for (bar = PCI_BASE_ADDRESS_1; bar <= PCI_BASE_ADDRESS_2; bar += 4) {
        early_write_config_dword(hose, hose->first_busno,
                     PCI_FUNC(hose->first_busno), bar,
                     0x00000000);
        early_read_config_dword(hose, hose->first_busno,
                    PCI_FUNC(hose->first_busno), bar,
                    &bar_response);
    }

    hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

    ppc_md.pci_swizzle = common_swizzle;
    ppc_md.pci_map_irq = ppc440epx_map_irq;
}

static
void __init europa_setup_pci(void)
{
	void *pci_reg_base;
	void *pci_cfg_base;
	unsigned long memory_size;

	memory_size = ppc_md.find_end_of_memory();

	pci_reg_base = ioremap64(PPC440_PCIL0_BASE, PPC440_PCIL0_SIZE);
	pci_cfg_base = ioremap64(PPC440_PCI_CFGREGS_BASE, 64);

    PCI_CFG_OUT(PPC440_PCI_CFGA_OFFSET, 0x80000000 | (PCI_COMMAND & 0xfc));
    PCI_CFG_OUT(PPC440_PCI_CFGD_OFFSET,
            (PCI_CFG_IN(PPC440_PCI_CFGD_OFFSET) |
             PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER));

    /* Disable region first */
    PCI_WRITEL(0, PPC440_PCIL0_PMM0MA);

    /* PLB starting addr: 0x0000000180000000 */
    PCI_WRITEL(PPC440_PCI_PHY_MEM_BASE, PPC440_PCIL0_PMM0LA);
    /* PCI start addr, 0x80000000 (PCI Address) */
    PCI_WRITEL(PPC440_PCI_MEM_BASE, PPC440_PCIL0_PMM0PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM0PCIHA);

    /* Enable no pre-fetch, enable region */
    PCI_WRITEL(((0xffffffff -
             (PPC440_PCI_UPPER_MEM - PPC440_PCI_MEM_BASE)) | 0x01),
           PPC440_PCIL0_PMM0MA);

    /* Disable region one */
    PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1LA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1PCIHA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);

    /* Disable region two */
    PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2LA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2PCIHA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);

    /* Now configure the PCI->PLB windows, we only use PTM1
     *
     * For Inbound flow, set the window size to all available memory
     * This is required because if size is smaller,
     * then Eth/PCI DD would fail as PCI card not able to access
     * the memory allocated by DD.
     */

    PCI_WRITEL(0, PPC440_PCIL0_PTM1MS); /* disabled region 1 */
    PCI_WRITEL(0, PPC440_PCIL0_PTM1LA); /* begin of address map */

    memory_size = 1 << fls(memory_size - 1);

    /* Size low + Enabled */
    PCI_WRITEL((0xffffffff - (memory_size - 1)) | 0x1, PPC440_PCIL0_PTM1MS);

    eieio();
    iounmap(pci_reg_base);
    iounmap(pci_cfg_base);
}

static
void __init europa_setup_hose(void)
{
    unsigned int bar_response, bar;
    struct pci_controller *hose;

	/* disabled for Europa, if not in standalone mode */
	if (silkworm_pci_standalone_scan == 0) {
		printk(KERN_INFO "Silkworm: Suppressing PCI init code for Europa SWBD88\n");
		return;
	}

    europa_setup_pci();

    hose = pcibios_alloc_controller();

    if (!hose)
        return;

    hose->first_busno = 0;
    hose->last_busno = 0xff;

    hose->pci_mem_offset = PPC440_PCI_MEM_OFFSET;

    pci_init_resource(&hose->io_resource,
              PPC440_PCI_LOWER_IO,
              PPC440_PCI_UPPER_IO,
              IORESOURCE_IO,
              "PCI host bridge");
    pci_init_resource(&hose->mem_resources[0],
              PPC440_PCI_LOWER_MEM,
              PPC440_PCI_UPPER_MEM,
              IORESOURCE_MEM,
              "PCI host bridge");

    ppc_md.pci_exclude_device = ppc440epx_exclude_device;

    hose->io_space.start = PPC440_PCI_LOWER_IO;
    hose->io_space.end = PPC440_PCI_UPPER_IO;
    hose->mem_space.start = PPC440_PCI_LOWER_MEM;
    hose->mem_space.end = PPC440_PCI_UPPER_MEM;
    isa_io_base =
        (unsigned long)ioremap64(PPC440_PCI_IO_BASE, PPC440_PCI_IO_SIZE);
    hose->io_base_virt = (void *)isa_io_base;

    setup_indirect_pci(hose, PPC440_PCI_CFGA_PLB32, PPC440_PCI_CFGD_PLB32);
    hose->set_cfg_type = 1;

    /* Zero config bars */
    for (bar = PCI_BASE_ADDRESS_1; bar <= PCI_BASE_ADDRESS_2; bar += 4) {
        early_write_config_dword(hose, hose->first_busno,
                     PCI_FUNC(hose->first_busno), bar,
                     0x00000000);
        early_read_config_dword(hose, hose->first_busno,
                    PCI_FUNC(hose->first_busno), bar,
                    &bar_response);
    }

    hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

    ppc_md.pci_swizzle = common_swizzle;
    ppc_md.pci_map_irq = ppc440epx_map_irq;
}



static
void __init spike_bp_setup_pci(void)
{
    void *pci_reg_base;
    void *pci_cfg_base;
    unsigned long memory_size;

    memory_size = ppc_md.find_end_of_memory();

    pci_reg_base = ioremap64(PPC440_PCIL0_BASE, PPC440_PCIL0_SIZE);
    pci_cfg_base = ioremap64(PPC440_PCI_CFGREGS_BASE, 64);

    PCI_CFG_OUT(PPC440_PCI_CFGA_OFFSET, 0x80000000 | (PCI_COMMAND & 0xfc));
    PCI_CFG_OUT(PPC440_PCI_CFGD_OFFSET,
            (PCI_CFG_IN(PPC440_PCI_CFGD_OFFSET) |
             PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER));

    /* Disable region first */
    PCI_WRITEL(0, PPC440_PCIL0_PMM0MA);

    /* PLB starting addr: 0x0000000180000000 */
    PCI_WRITEL(PPC440_PCI_PHY_MEM_BASE, PPC440_PCIL0_PMM0LA);
    /* PCI start addr, 0x80000000 (PCI Address) */
    PCI_WRITEL(PPC440_PCI_MEM_BASE, PPC440_PCIL0_PMM0PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM0PCIHA);

    /* Enable no pre-fetch, enable region */
    PCI_WRITEL(((0xffffffff -
             (PPC440_PCI_UPPER_MEM - PPC440_PCI_MEM_BASE)) | 0x01),
           PPC440_PCIL0_PMM0MA);

    /* Disable region one */
    PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1LA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1PCIHA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);

    /* Disable region two */
    PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2LA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2PCILA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2PCIHA);
    PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);

    /* Now configure the PCI->PLB windows, we only use PTM1
     *
     * For Inbound flow, set the window size to all available memory
     * This is required because if size is smaller,
     * then Eth/PCI DD would fail as PCI card not able to access
     * the memory allocated by DD.
     */

    PCI_WRITEL(0, PPC440_PCIL0_PTM1MS); /* disabled region 1 */
    PCI_WRITEL(0, PPC440_PCIL0_PTM1LA); /* begin of address map */

    memory_size = 1 << fls(memory_size - 1);

    /* Size low + Enabled */
    PCI_WRITEL((0xffffffff - (memory_size - 1)) | 0x1, PPC440_PCIL0_PTM1MS);

    eieio();
    iounmap(pci_reg_base);
    iounmap(pci_cfg_base);
}

static
void __init spike_bp_setup_hose(void)
{
    unsigned int bar_response, bar;
    struct pci_controller *hose;

    /* disabled for Spike BP */
    printk(KERN_INFO "Silkworm: Suppressing PCI init code for Spike BP SWBD85\n");
    return;

    spike_bp_setup_pci();

    hose = pcibios_alloc_controller();

    if (!hose)
        return;

    hose->first_busno = 0;
    hose->last_busno = 0xff;

    hose->pci_mem_offset = PPC440_PCI_MEM_OFFSET;

    pci_init_resource(&hose->io_resource,
              PPC440_PCI_LOWER_IO,
              PPC440_PCI_UPPER_IO,
              IORESOURCE_IO,
              "PCI host bridge");
    pci_init_resource(&hose->mem_resources[0],
              PPC440_PCI_LOWER_MEM,
              PPC440_PCI_UPPER_MEM,
              IORESOURCE_MEM,
              "PCI host bridge");

    ppc_md.pci_exclude_device = ppc440epx_exclude_device;

    hose->io_space.start = PPC440_PCI_LOWER_IO;
    hose->io_space.end = PPC440_PCI_UPPER_IO;
    hose->mem_space.start = PPC440_PCI_LOWER_MEM;
    hose->mem_space.end = PPC440_PCI_UPPER_MEM;
    isa_io_base =
        (unsigned long)ioremap64(PPC440_PCI_IO_BASE, PPC440_PCI_IO_SIZE);
    hose->io_base_virt = (void *)isa_io_base;

    setup_indirect_pci(hose, PPC440_PCI_CFGA_PLB32, PPC440_PCI_CFGD_PLB32);
    hose->set_cfg_type = 1;

    /* Zero config bars */
    for (bar = PCI_BASE_ADDRESS_1; bar <= PCI_BASE_ADDRESS_2; bar += 4) {
        early_write_config_dword(hose, hose->first_busno,
                     PCI_FUNC(hose->first_busno), bar,
                     0x00000000);
        early_read_config_dword(hose, hose->first_busno,
                    PCI_FUNC(hose->first_busno), bar,
                    &bar_response);
    }

    hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

    ppc_md.pci_swizzle = common_swizzle;
    ppc_md.pci_map_irq = ppc440epx_map_irq;
}


static void __init falcon2_setup_pci(void)
{
	void *pci_reg_base;
	void *pci_cfg_base;
	unsigned long memory_size;

	memory_size = ppc_md.find_end_of_memory();

	pci_reg_base = ioremap64(PPC440_PCIL0_BASE, PPC440_PCIL0_SIZE);
	pci_cfg_base = ioremap64(PPC440_PCI_CFGREGS_BASE, 64);

	PCI_CFG_OUT(PPC440_PCI_CFGA_OFFSET, 0x80000000 | (PCI_COMMAND & 0xfc));
	PCI_CFG_OUT(PPC440_PCI_CFGD_OFFSET,
		    (PCI_CFG_IN(PPC440_PCI_CFGD_OFFSET) |
		     PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER));

	/* Disable region first */
	PCI_WRITEL(0, PPC440_PCIL0_PMM0MA);

	/* PLB starting addr: 0x0000000180000000 */
	PCI_WRITEL(PPC440_PCI_PHY_MEM_BASE, PPC440_PCIL0_PMM0LA);

	/* PCI start addr, 0x80000000 (PCI Address) */
	PCI_WRITEL(PPC440_PCI_MEM_BASE, PPC440_PCIL0_PMM0PCILA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM0PCIHA);

	/* Enable no pre-fetch, enable region */
	PCI_WRITEL(((0xffffffff -
		     (PPC440_PCI_UPPER_MEM - PPC440_PCI_MEM_BASE)) | 0x01),
		   PPC440_PCIL0_PMM0MA);

	/* Disable region one */
	PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1LA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1PCILA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1PCIHA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM1MA);

	/* Disable region two */
	PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2LA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2PCILA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2PCIHA);
	PCI_WRITEL(0, PPC440_PCIL0_PMM2MA);

	/* Now configure the PCI->PLB windows, we only use PTM1
	 *
	 * For Inbound flow, set the window size to all available memory
	 * This is required because if size is smaller,
	 * then Eth/PCI DD would fail as PCI card not able to access
	 * the memory allocated by DD.
	 */

	PCI_WRITEL(0, PPC440_PCIL0_PTM1MS);	/* disabled region 1 */
	PCI_WRITEL(0, PPC440_PCIL0_PTM1LA);	/* begin of address map */

	memory_size = 1 << fls(memory_size - 1);

	/* Size low + Enabled */
	PCI_WRITEL((0xffffffff - (memory_size - 1)) | 0x1, PPC440_PCIL0_PTM1MS);

	eieio();
	iounmap(pci_reg_base);
	iounmap(pci_cfg_base);
}

static void __init falcon2_setup_hose(void)
{
	unsigned int bar_response, bar;
	struct pci_controller *hose;

	falcon2_setup_pci();

	hose = pcibios_alloc_controller();

	if (!hose)
		return;

	hose->first_busno = 0;
	hose->last_busno = 0xff;

	hose->pci_mem_offset = PPC440_PCI_MEM_OFFSET;

	pci_init_resource(&hose->io_resource,
			  PPC440_PCI_LOWER_IO,
			  PPC440_PCI_UPPER_IO,
			  IORESOURCE_IO,
			  "PCI host bridge");

	pci_init_resource(&hose->mem_resources[0],
			  PPC440_PCI_LOWER_MEM,
			  PPC440_PCI_UPPER_MEM,
			  IORESOURCE_MEM,
			  "PCI host bridge");

	ppc_md.pci_exclude_device = ppc440epx_exclude_device;

	hose->io_space.start = PPC440_PCI_LOWER_IO;
	hose->io_space.end = PPC440_PCI_UPPER_IO;
	hose->mem_space.start = PPC440_PCI_LOWER_MEM;
	hose->mem_space.end = PPC440_PCI_UPPER_MEM;
	isa_io_base =
		(unsigned long)ioremap64(PPC440_PCI_IO_BASE, PPC440_PCI_IO_SIZE);
	hose->io_base_virt = (void *)isa_io_base;

	setup_indirect_pci(hose, PPC440_PCI_CFGA_PLB32, PPC440_PCI_CFGD_PLB32);
	hose->set_cfg_type = 1;

	/* Zero config bars */
	for (bar = PCI_BASE_ADDRESS_1; bar <= PCI_BASE_ADDRESS_2; bar += 4) {
		early_write_config_dword(hose, hose->first_busno,
					 PCI_FUNC(hose->first_busno), bar,
					 0x00000000);
		early_read_config_dword(hose, hose->first_busno,
					PCI_FUNC(hose->first_busno), bar,
					&bar_response);
	}

	hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

	ppc_md.pci_swizzle = common_swizzle;
	ppc_md.pci_map_irq = ppc440epx_map_irq;
}

static void __init ppc440epx_early_serial_map(void)
{
	struct uart_port port;

	/* Setup ioremapped serial port access */
	memset(&port, 0, sizeof(port));

	if (silkworm.board == SWBD130) { /* SH3 use UART0 as console */
		port.membase = ioremap64(PPC440EPX_UART0_ADDR, 8);
		port.irq = UART0_INT;
		port.uartclk = clocks.uart0;
		port.regshift = 0;
		port.iotype = SERIAL_IO_MEM;
		port.flags = ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST;
		port.line = 0;
	
		if (early_serial_setup(&port) != 0)
			printk("Early serial init of port 0 failed\n");
	
#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
		/* Configure debug serial access */
		gen550_init(0, &port);
#endif
	
		port.membase = ioremap64(PPC440EPX_UART1_ADDR, 8);
		port.irq = UART1_INT;
		port.uartclk = clocks.uart1;
		port.line = 1;
	} else {
		port.membase = ioremap64(PPC440EPX_UART1_ADDR, 8);
		port.irq = UART1_INT;
		port.uartclk = clocks.uart1;
		port.regshift = 0;
		port.iotype = SERIAL_IO_MEM;
		port.flags = ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST;
		port.line = 0;
	
		if (early_serial_setup(&port) != 0)
			printk("Early serial init of port 0 failed\n");
	
#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
		/* Configure debug serial access */
		gen550_init(0, &port);
#endif
	
		port.membase = ioremap64(PPC440EPX_UART0_ADDR, 8);
		port.irq = UART0_INT;
		port.uartclk = clocks.uart0;
		port.line = 1;
	}
		if (early_serial_setup(&port) != 0)
			printk("Early serial init of port 1 failed\n");
	
#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
		/* Configure debug serial access */
		gen550_init(1, &port);
#endif
	port.membase = ioremap64(PPC440EPX_UART2_ADDR, 8);
	port.irq = UART2_INT;
	port.uartclk = clocks.uart2;
	port.line = 2;

	if (early_serial_setup(&port) != 0)
		printk("Early serial init of port 2 failed\n");

#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	/* Configure debug serial access */
	gen550_init(2, &port);
#endif

	port.membase = ioremap64(PPC440EPX_UART3_ADDR, 8);
	port.irq = UART3_INT;
	port.uartclk = clocks.uart3;
	port.line = 3;

	if (early_serial_setup(&port) != 0)
		printk("Early serial init of port 3 failed\n");
}

#define	I2C0	0
#define	I2C1	1
static void __init ppc440epx_i2c_fastmode(int unit)
{
	struct ocp_def *def = ocp_get_one_device(OCP_VENDOR_IBM,
					OCP_FUNC_IIC, unit);

	if (def) {
		struct ocp_func_iic_data *iic_data = def->additions;
		if (iic_data) {
			iic_data->fast_mode = 1; /* Set I2C clock to 400 kHz */
		}
	}
}

static void __init 
ppc440epx_setup_arch(void)
{

	/* Set mac_addr for each EMAC */
	silkworm440epx_set_emacdata();

	/* parm1 = sys clock is OK , parm 2 ser_clock to be checked */
	ibm440gx_get_clocks(&clocks, 33333333, 6 * 1843200);
	ocp_sys_info.opb_bus_freq = clocks.opb;
	ocp_sys_info.plb_bus_freq = clocks.plb;

	/* init to some ~sane value until calibrate_delay() runs */
    loops_per_jiffy = 50000000/HZ;

	/* Setup PCI host bridge */
	switch (silkworm.board) {
		case SWBD71: /* Tomahawk */
		case SWBD66: /* Spirit same as Tomahawk */
		case SWBD92: /* Gladius same as Tomahawk */
		case SWBD109: /* Stinger same as Tomahawk */
		case SWBD118: /* Tomtoo same as Tomahawk */
			tomahawk_setup_hose();
			ppc4xx_uic_ext_irq_cfg = tomahawk_irq_initsenses;
			break;

		case SWBD73: /* Blazer3 (Dirk) */
			tomahawk_setup_hose();
			ppc4xx_uic_ext_irq_cfg = blazer3_irq_initsenses;
			break;
		
		case SWBD75: /* Superhawk2 */
			tomahawk_setup_hose();
			ppc4xx_uic_ext_irq_cfg = superhawk2_irq_initsenses;
			break;

		case SWBD72:
		case SWBD82:
			/* Bulova and Utah */
			bulova_setup_hose();
			ppc4xx_uic_ext_irq_cfg = bulova_irq_initsenses;
			break;
                case SWBD83:
                        spike_cp_setup_hose();
                        ppc4xx_uic_ext_irq_cfg = spike_cp_irq_initsenses;
                        break;

		case SWBD85:
			spike_bp_setup_hose();
			ppc4xx_uic_ext_irq_cfg = spike_bp_irq_initsenses;
			break;

		case SWBD78:
			leda_setup_hose();
			ppc4xx_uic_ext_irq_cfg = leda_irq_initsenses;
			break;

		case SWBD88: /* Europa */
			europa_setup_hose();
			ppc4xx_uic_ext_irq_cfg = europa_irq_initsenses;
			break;
			
		case SWBD86:
			/* Falcon2 */
			falcon2_setup_hose();
			ppc4xx_uic_ext_irq_cfg = falcon2_irq_initsenses;
			break;

		case SWBD87: /* Kestrel2 */
			tomahawk_setup_hose();
			ppc4xx_uic_ext_irq_cfg = kestrel2_irq_initsenses;
			break;

		case SWBD117: /* Pharos */
			tomahawk_setup_hose();
			ppc4xx_uic_ext_irq_cfg = pharos_irq_initsenses;
			/*
			 * Set I2C1 clock to 400 kHz. I2C0 clock remains
			 * unchanged at 100 kHz as it is unsure whether
			 * all the devices on I2C0 are 400kHz compatible
			 */
			ppc440epx_i2c_fastmode(I2C1);
			break;

		case SWBD124: /* Dragon2 */
			tomahawk_setup_hose();
			ppc4xx_uic_ext_irq_cfg = dragon2_irq_initsenses;
			break;

		case SWBD125:  /*Dragon2T*/
		case SWBD134:  /*Qilin*/
		        tomahawk_setup_hose();
			ppc4xx_uic_ext_irq_cfg = dragon2t_irq_initsenses;
			break;

		case SWBD129: /* Bulova2 */
			tomahawk_setup_hose();
			ppc4xx_uic_ext_irq_cfg = bulova2_irq_initsenses;
			/*
			 * Set I2C1 clock to 400 kHz. I2C0 clock remains
			 * unchanged at 100 kHz as it is unsure whether
			 * all the devices on I2C0 are 400kHz compatible
			 */
			//ppc440epx_i2c_fastmode(I2C1);
			break;
			
		case SWBD149: /* Kestrel3 */
			tomahawk_setup_hose();
			ppc4xx_uic_ext_irq_cfg = kestrel3_irq_initsenses;
			/*
			 * Set I2C1 clock to 400 kHz. I2C0 clock remains
			 * unchanged at 100 kHz as it is unsure whether
			 * all the devices on I2C0 are 400kHz compatible
			 */
			//ppc440epx_i2c_fastmode(I2C1);
			break;
			
		case SWBD130: /* Superhawk3 */
			tomahawk_setup_hose();
			ppc4xx_uic_ext_irq_cfg = superhawk3_irq_initsenses;
			/*
			 * Set I2C1 clock to 400 kHz. I2C0 clock remains
			 * unchanged at 100 kHz as it is unsure whether
			 * all the devices on I2C0 are 400kHz compatible
			 */
			//ppc440epx_i2c_fastmode(I2C1);
			break;
		case SWBD150: /* Falcon3 */
			tomahawk_setup_hose();
			ppc4xx_uic_ext_irq_cfg = falcon3_irq_initsenses;
			/*
			 * Set I2C1 clock to 400 kHz. I2C0 clock remains
			 * unchanged at 100 kHz as it is unsure whether
			 * all the devices on I2C0 are 400kHz compatible
			 */
			//ppc440epx_i2c_fastmode(I2C1);
			break;

  		default:
			printk(KERN_WARNING
					"ppc440epx_setup_arch(): PCI host bridge/UIC setup unknown SWBD %d\n",
					silkworm.board);

			break;
	}

	cpld_init();

#ifdef CONFIG_BLK_DEV_INITRD
#ifdef CONFIG_ROOT_NFS
		ROOT_DEV = Root_NFS;
#else
		ROOT_DEV = Root_HDA1;
#endif
#endif

	ppc440epx_early_serial_map();

	/* Identify the system */
	printk("Brocade Silkworm port (C) 2007 DENX Software Engineering (sr@denx.de)\n");
}

static void __init ppc440epx_init_irq(void)
{
	ppc4xx_pic_init();
}

/*
 * superhawk2_reset()
 *
 * Description:
 *   Satisfies a Superhawk2 requirement. Sets a bit in the FPGA "CMC support"
 *   register. This bit tells the chassis management controller that some user
 *   has intentionally reset the switch.
 */
static void
superhawk2_reset(void)
{
        volatile char *cpldp = cpld_get_base_addr();
        if (cpldp) {
                enum { SWITCH_INTENDED_RESET = 0x10 };
                cpldp += SUPERHAWK_CMC_SUP_REG;
                writeb(SWITCH_INTENDED_RESET | readb(cpldp), cpldp);
        } else {
                printk(KERN_WARNING "%s: cpld base is NULL\n", __FUNCTION__);
        }
}

static void
superhawk2_restart(char *cmd)
{
        if (cmd)
                printk("%s\n", cmd);
        superhawk2_reset();
        abort();
}

static void
superhawk2_halt(void)
{
        superhawk2_reset();
        abort();
}

static void
superhawk2_power_off(void)
{
        superhawk2_reset();
        cpld_power_off();
}

static void
cpld_power_off_spike(void)
{
	volatile char *cpld_base, *cpld_reset_enable, *cpld_reset_shutdown;
	unsigned char read_l;	
	cpld_base = (char *) ioremap(PULSAR_CPLD_BASE_ADDR, 32);
	if (cpld_base) {

		cpld_reset_enable = cpld_base + SPIKE_SYS_RESET_ENABLE_REG;
		cpld_reset_shutdown = cpld_base + SPIKE_SYS_RESET_CONTROL_REG;

		/*
		 * Arm for system shutdown.
		 */
		writeb(SPIKE_SYSTEM_RESET_ENABLE, cpld_reset_enable);
		
		/*
		 * Shutdown the system.
		 */
		writeb(SPIKE_SYSTEM_SHUTDOWN_ENABLE, cpld_reset_shutdown);
	} else {
		printk(KERN_CRIT "%s: ioremap failed\n", __FUNCTION__);
	}

}


static void
ppc440_power_off(void)
{
	extern void ppc405wd_update_timer_shutdown(void);

	local_irq_disable();
	for(;;) {
	  ppc405wd_update_timer_shutdown();
	}
}

static void
safe_power_off(void)
{
	cpld_power_off();
	/* if the FPGA doesn't implement halt, prevent watchdog panic */
	ppc440_power_off();
}

/*
 * bladecenter_reset()
 *
 * Description:
 *   Sets a bit in the FPGA "Software Restarts" register. At the next start-up,
 *   the platform driver can check this bit to know whether the reboot was
 *   caused by power-on or not. If this bit is clear, power must have been
 *   turned off and then on.
 *
 *   The motivation is the IBM BladeCenter requirement to maintain a counter
 *   of PD2 cycles. "PD2" means Power Domain 2, which is the power domain for
 *   the microprocessor, among other things.
 */
static void
bladecenter_reset(void)
{
	volatile char *p = cpld_get_base_addr();
	if (p) {
		enum {
			FPGA_REG_BLADECENTER_RESET = 0xf6,
			SW_INTENTIONAL_RESTART = 1,
		};
		writeb(0, p + STEALTH_SYS_STATUS_LED_CNTRL_REG); /* LEDs off */
		p += FPGA_REG_BLADECENTER_RESET;
		writeb(SW_INTENTIONAL_RESTART | readb(p), p);
	} else {
		printk(KERN_WARNING "%s: cpld base is NULL\n", __FUNCTION__);
	}
}

static void
bladecenter_restart(char *cmd)
{
	if (cmd)
		printk("%s\n", cmd);
	bladecenter_reset();
	abort();
}

static void
bladecenter_halt(void)
{
	bladecenter_reset();
	abort();
}

static void
bladecenter_power_off(void)
{
	bladecenter_reset();
	safe_power_off();
}

/* The PCI SERR interrupt */
static irqreturn_t pci_serr_handler(int irq, void *dev_instance, struct pt_regs *regs)
{
    printk("*** PCI SERR ***\n");
    abort();
    return IRQ_HANDLED;
}

/* Pharos intialize */
static void pharos_init(void)
{
#define EXT_PCI_SERR	9
	/* Set the PCI SERR interrupt */
    request_irq(EXT_PCI_SERR, pci_serr_handler, SA_INTERRUPT, "PCI SERR", 0);
}

void __init platform_init(unsigned long r3, unsigned long r4,
			  unsigned long r5, unsigned long r6, unsigned long r7)
{

	ibm44x_platform_init(r3, r4, r5, r6, r7);

	ppc_md.setup_arch = ppc440epx_setup_arch;
	ppc_md.show_cpuinfo = ppc440epx_show_cpuinfo;
	ppc_md.find_end_of_memory = ppc440epx_find_end_of_memory;
	ppc_md.get_irq = NULL;		/* Set in ppc4xx_pic_init() */

	ppc_md.calibrate_decr = ppc440epx_calibrate_decr;
	ppc_md.time_init = NULL;
	ppc_md.set_rtc_time = NULL;
	ppc_md.get_rtc_time = NULL;

	ppc_md.init_IRQ = ppc440epx_init_irq;


#ifdef CONFIG_KGDB
	ppc_md.early_serial_map = ppc440epx_early_serial_map;
#endif

#ifdef CONFIG_PPC405_WDT
        ppc_md.heartbeat = ppc4xx_wdt_heartbeat;
#endif
        ppc_md.heartbeat_count = 0;

#if defined(CONFIG_BRCD_CPLD_IDE)
	switch (silkworm.board) {
		case SWBD71:
		case SWBD66:
		case SWBD73:
		case SWBD75:
		case SWBD87:
		case SWBD92:
		case SWBD109:
		case SWBD117:
		case SWBD118:
		case SWBD129:
		case SWBD130:
		case SWBD124:
		case SWBD125:
		case SWBD134:
		case SWBD149:
		case SWBD150:
			ppc_ide_md.ide_init_hwif = cpld_tomahawk_ide_init_hwif_ports;
			break;

		case SWBD72:
		case SWBD82:
			ppc_ide_md.ide_init_hwif = cpld_bulova_ide_init_hwif_ports;
			break;

		case SWBD78:
		case SWBD88: /* Europa */
			ppc_ide_md.ide_init_hwif = cpld_leda_ide_init_hwif_ports;
			break;
		case SWBD85:
			ppc_ide_md.ide_init_hwif = cpld_spike_bp_ide_init_hwif_ports;
			break;
                case SWBD83: /* Spike CP */
                        ppc_ide_md.ide_init_hwif = cpld_spike_cp_ide_init_hwif_ports;
                        break;          

		case SWBD86:
			ppc_ide_md.ide_init_hwif = cpld_falcon2_ide_init_hwif_ports;
			break;

		default:
			printk(KERN_WARNING "cpld_ide_init_hwif_ports: The platform is "
					"not supported\n");
			break;
	}
#endif

	/*
	 * We need to supply memhole routine defined in silkworm_feature.c,
	 * and a routine to power-down the board for use with sysShutdown.
	 */
	switch (silkworm.board) {
		case SWBD71:
		case SWBD72:
		case SWBD82:
		case SWBD66:
		case SWBD73:
		case SWBD78:
		case SWBD88:
		case SWBD86:
		case SWBD87:
		case SWBD92:
		case SWBD109:
		case SWBD118:
		case SWBD129:
		case SWBD124:
		case SWBD125:
		case SWBD134:
		case SWBD149:
		case SWBD150:
			ppc_md.feature_call = ppc44x_do_feature_call;
			ppc_md.power_off = safe_power_off;
			break;
		case SWBD83:
			ppc_md.feature_call = ppc44x_do_feature_call;
			ppc_md.power_off = cpld_power_off_spike;
			break;
		case SWBD75:
			ppc_md.feature_call = ppc44x_do_feature_call;
			ppc_md.power_off = superhawk2_power_off;
			ppc_md.halt = superhawk2_halt;
			ppc_md.restart = superhawk2_restart;
			break;
		case SWBD117:
			ppc_md.feature_call = ppc44x_do_feature_call;
			ppc_md.power_off = bladecenter_power_off;
			ppc_md.halt = bladecenter_halt;
			ppc_md.restart = bladecenter_restart;
		case SWBD130:
			ppc_md.feature_call = ppc44x_do_feature_call;
			ppc_md.power_off = superhawk2_power_off;
			ppc_md.halt = superhawk2_halt;
			ppc_md.restart = superhawk2_restart;
			break;
		default:
			ppc_md.feature_call = ppc44x_do_feature_call;
			ppc_md.power_off = ppc440_power_off;
			ppc_md.halt = ppc440_power_off;
			break;
	}

	/* To support the PCI SERR interrupt */
	switch (silkworm.board) {
		case SWBD117: /* Pharos */
			ppc_md.init = pharos_init;
			break;
		default:
			break;
	}

	if (ppc_md.progress)
		ppc_md.progress("silkworm440epx_init(): exit", 0);
		
	return;

}

#if defined(CONFIG_BRCD_CPLD_IDE)
static void
cpld_tomahawk_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port, unsigned long ctrl_port, int *irq)
{
    unsigned long  *p;
    unsigned int uicdcr;
    int i;
    static uint8_t cpld_data, *cpld_base, *ata_base;

    switch (silkworm.board) {
	case SWBD66:
	case SWBD71:
	case SWBD73:
	case SWBD75:
	case SWBD87:
	case SWBD92:
	case SWBD109:
	case SWBD117:
	case SWBD118:
	case SWBD129:
	case SWBD130:
	case SWBD124:
	case SWBD125:
	case SWBD134:
	case SWBD149:   /* Kestrel3 */
	case SWBD150:
		break;
	default:
        BUG();
    }

    p = hw->io_ports;
    *p = 0;
    if (irq)
        *irq = 0;

    /* IS this correct?  How else do we register only ide0 ? */
    if(ata_base) {
        return;
	}

    if (data_port != 0)
        return;

    if ((ata_base = (void*) ioremap64(PULSAR_CPLD_IDE_BASE_ADDR,
            (unsigned long)PULSAR_CPLD_IDE_SIZE * 4)) == NULL) {
        printk(KERN_WARNING "cpld_ide_init_hwif_ports: failed ioremap "
                "of task file\n");
        return;
    }
    /* Bootprom capability not checked for marathon */

    /* assign the hw ports */
    //hack - the ide driver uses inb() to do the read - inb() 
    // adds _IO_BASE to the address before reading - need to subtract 
    // that off here, or get the ide driver
    //to use readb instead (but still work with PCI ide controllers...)

    for (i = 0; i <= IDE_STATUS_OFFSET; i++) {
        *p++ = (unsigned long)
            ata_base + ((PULSAR_REGISTER_FILE1_REG + i) << 1) - _IO_BASE;
    }
    hw->io_ports[IDE_CONTROL_OFFSET] =
        (unsigned long) ata_base + (PULSAR_REGISTER_FILE2_REG << 1) - _IO_BASE;

    if (irq)
        *irq = 64;      //todo - make this a define IRQ5

    ide_hwifs[data_port].noprobe = 0;

    printk(KERN_WARNING, "FIXME ---- Silkworm CE-2 CPLD ATA "
            "interface configured [CPLD version %d]\n", 0x1f);
}

static void
cpld_bulova_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port, unsigned long ctrl_port, int *irq)
{
    unsigned long  *p;
    unsigned int uicdcr;
    int i;
    static uint8_t cpld_data, *cpld_base, *ata_base;

    if ((silkworm.board != SWBD72) && (silkworm.board != SWBD82))
        BUG();

    p = hw->io_ports;
    *p = 0;
    if (irq)
        *irq = 0;

    /* IS this correct?  How else do we register only ide0 ? */
    if(ata_base) {
        return;
	}

    if (data_port != 0)
        return;

    if ((ata_base = (void*) ioremap64(PULSAR_CPLD_IDE_BASE_ADDR,
            (unsigned long)PULSAR_CPLD_IDE_SIZE * 4)) == NULL) {
        printk(KERN_WARNING "cpld_ide_init_hwif_ports: failed ioremap "
                "of task file\n");
        return;
    }
    /* Bootprom capability not checked for marathon */

    /* assign the hw ports */
    //hack - the ide driver uses inb() to do the read - inb() 
    // adds _IO_BASE to the address before reading - need to subtract 
    // that off here, or get the ide driver
    //to use readb instead (but still work with PCI ide controllers...)

    for (i = 0; i <= IDE_STATUS_OFFSET; i++) {
        *p++ = (unsigned long)
            ata_base + ((PULSAR_REGISTER_FILE1_REG + i) << 1) - _IO_BASE;
    }
    hw->io_ports[IDE_CONTROL_OFFSET] =
        (unsigned long) ata_base + (PULSAR_REGISTER_FILE2_REG << 1) - _IO_BASE;

    if (irq)
        *irq = 64;      //todo - make this a define IRQ5

    ide_hwifs[data_port].noprobe = 0;

    printk(KERN_WARNING, "FIXME ---- Silkworm CE-2 CPLD ATA "
            "interface configured [CPLD version %d]\n", 0x1f);
}

static
void cpld_leda_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port,
				unsigned long ctrl_port, int *irq)
{
	int i;
	unsigned long  *p;
	unsigned int uicdcr;
	static uint8_t cpld_data, *cpld_base, *ata_base;

    if ((silkworm.board != SWBD78) && (silkworm.board != SWBD88)) {
		BUG();
	}

	p = hw->io_ports;
	*p = 0;
	if (irq) {
		*irq = 0;
	}

	if (ata_base) {
		return;
	}

	if (data_port != 0) {
		return;
	}

	if ((ata_base = (void*) ioremap64(PULSAR_CPLD_IDE_BASE_ADDR,
			(unsigned long)PULSAR_CPLD_IDE_SIZE * 4)) == NULL) {
		printk(KERN_WARNING "cpld_ide_init_hwif_ports: failed ioremap "
				"of task file\n");
		return;
	}

	/* assign the hw ports */
	//hack - the ide driver uses inb() to do the read - inb()
	// adds _IO_BASE to the address before reading - need to subtract
	// that off here, or get the ide driver
	//to use readb instead (but still work with PCI ide controllers...)

	for (i = 0; i <= IDE_STATUS_OFFSET; i++) {
		*p++ = (unsigned long)
		ata_base + ((PULSAR_REGISTER_FILE1_REG + i) << 1) - _IO_BASE;
	}

	hw->io_ports[IDE_CONTROL_OFFSET] =
		(unsigned long) ata_base + (PULSAR_REGISTER_FILE2_REG << 1) - _IO_BASE;

	if (irq) {
		*irq = 64;
	}

	ide_hwifs[data_port].noprobe = 0;

	printk(KERN_WARNING, "FIXME ---- Silkworm CE-2 CPLD ATA "
			"interface configured [CPLD version %d]\n", 0x1f);
}

static
void cpld_spike_bp_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port,
                unsigned long ctrl_port, int *irq)
{
    int i;
    unsigned long  *p;
    unsigned int uicdcr;
    static uint8_t cpld_data, *cpld_base, *ata_base;

    if (silkworm.board != SWBD85) {
        BUG();
    }

    p = hw->io_ports;
    *p = 0;
    if (irq) {
        *irq = 0;
    }

    if (ata_base) {
        return;
    }

    if (data_port != 0) {
        return;
    }

    if ((ata_base = (void*) ioremap64(PULSAR_CPLD_IDE_BASE_ADDR,
            (unsigned long)PULSAR_CPLD_IDE_SIZE * 4)) == NULL) {
        printk(KERN_WARNING "cpld_ide_init_hwif_ports: failed ioremap "
                "of task file\n");
        return;
    }

    /* assign the hw ports */
    //hack - the ide driver uses inb() to do the read - inb()
    // adds _IO_BASE to the address before reading - need to subtract
    // that off here, or get the ide driver
    //to use readb instead (but still work with PCI ide controllers...)

    for (i = 0; i <= IDE_STATUS_OFFSET; i++) {
        *p++ = (unsigned long)
        ata_base + ((PULSAR_REGISTER_FILE1_REG + i) << 1) - _IO_BASE;
    }

    hw->io_ports[IDE_CONTROL_OFFSET] =
        (unsigned long) ata_base + (PULSAR_REGISTER_FILE2_REG << 1) - _IO_BASE;

    if (irq) {
        *irq = 64;
    }

    ide_hwifs[data_port].noprobe = 0;
    printk(KERN_WARNING, "FIXME ---- Silkworm CE-2 CPLD ATA "
            "interface configured [CPLD version %d]\n", 0x1f);
}


static
void cpld_spike_cp_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port,
                                unsigned long ctrl_port, int *irq)
{
        int i;
        unsigned long  *p;
        unsigned int uicdcr;
        static uint8_t cpld_data, *cpld_base, *ata_base;

        if (silkworm.board != SWBD83) {
              BUG();
        }

        p = hw->io_ports;
        *p = 0;
        if (irq) {
                *irq = 0;
        }

        if (ata_base) {
                return;
        }

        if (data_port != 0) {
                return;
        }

        if ((ata_base = (void*) ioremap64(PULSAR_CPLD_IDE_BASE_ADDR,
                        (unsigned long)PULSAR_CPLD_IDE_SIZE * 4)) == NULL) {
                printk(KERN_WARNING "cpld_spike_cp_init_hwif_ports: failed ioremap "
                                "of task file\n");
                return;
        }

        /* assign the hw ports */
        //hack - the ide driver uses inb() to do the read - inb()
        // adds _IO_BASE to the address before reading - need to subtract
        // that off here, or get the ide driver
        //to use readb instead (but still work with PCI ide controllers...)

        for (i = 0; i <= IDE_STATUS_OFFSET; i++) {
                *p++ = (unsigned long)
                ata_base + ((PULSAR_REGISTER_FILE1_REG + i) << 1) - _IO_BASE;
        }

        hw->io_ports[IDE_CONTROL_OFFSET] =
                (unsigned long) ata_base + (PULSAR_REGISTER_FILE2_REG << 1) - _IO_BASE;

        if (irq) {
                *irq = 64;
        }

        ide_hwifs[data_port].noprobe = 0;

        printk(KERN_WARNING, "FIXME ---- Silkworm CE-2 CPLD ATA "
                        "interface configured [CPLD version %d]\n", 0x1f);
}

static void
cpld_falcon2_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port, unsigned long ctrl_port, int *irq)
{
    unsigned long  *p;
    unsigned int uicdcr;
    int i;
    static uint8_t cpld_data, *cpld_base, *ata_base;

    if (silkworm.board != SWBD86)
        BUG();

    p = hw->io_ports;
    *p = 0;
    if (irq)
        *irq = 0;

    /* IS this correct?  How else do we register only ide0 ? */
    if(ata_base) {
        return;
	}

    if (data_port != 0)
        return;

    if ((ata_base = (void*) ioremap64(PULSAR_CPLD_IDE_BASE_ADDR,
            (unsigned long)PULSAR_CPLD_IDE_SIZE * 4)) == NULL) {
        printk(KERN_WARNING "cpld_ide_init_hwif_ports: failed ioremap "
                "of task file\n");
        return;
    }
    /* Bootprom capability not checked for marathon */

    /* assign the hw ports */
    //hack - the ide driver uses inb() to do the read - inb() 
    // adds _IO_BASE to the address before reading - need to subtract 
    // that off here, or get the ide driver
    //to use readb instead (but still work with PCI ide controllers...)

    for (i = 0; i <= IDE_STATUS_OFFSET; i++) {
        *p++ = (unsigned long)
            ata_base + ((PULSAR_REGISTER_FILE1_REG + i) << 1) - _IO_BASE;
    }
    hw->io_ports[IDE_CONTROL_OFFSET] =
        (unsigned long) ata_base + (PULSAR_REGISTER_FILE2_REG << 1) - _IO_BASE;

    if (irq)
        *irq = 64;      //todo - make this a define IRQ5

    ide_hwifs[data_port].noprobe = 0;

    printk(KERN_WARNING, "FIXME ---- Silkworm CE-2 CPLD ATA "
            "interface configured [CPLD version %d]\n", 0x1f);
}
#endif

/*
 * This routine disables the USB intrrupt.
 * returns 0 on success -1 otherwise
 */
int silkworm_mask_usb_intr()
{
    switch (silkworm.board) {
    	case SWBD71:	/* dummy for Tomahawk, no external irq for usb */
		case SWBD66:	/* Sprint */
		case SWBD78:     /* Leda */
		case SWBD88:	/* Europa */
		case SWBD85:	/* Spike-BP */
		case SWBD83:	/* Spike CP */
		case SWBD87:	/* Kestrel2 */
		case SWBD92:
		case SWBD109:
		case SWBD117:
		case SWBD118:
		case SWBD129:
		case SWBD130:
		case SWBD124:	/* Dragon2 */
		case SWBD125:   /* Dragon2T */
		case SWBD134:   /* Qilin */
		case SWBD149:   /* Kestrel3 */
		case SWBD150:
			break;

		default:
			break;
    }

	return 0;
}

/*
 * This routine enables the USB intrrupt.
 * returns 0 on success -1 otherwise
 */
int silkworm_unmask_usb_intr()
{
    switch (silkworm.board) {
    	case SWBD71:	/* dummy for Tomahawk, no external irq for usb */
		case SWBD66:	/* Sprint */
		case SWBD78:	/* Leda */
		case SWBD88:	/* Europa */
		case SWBD85:	/* Spike-BP */
		case SWBD83:	/* Spike CP */
		case SWBD87:	/* Kestrel2 */
		case SWBD92:
		case SWBD109:
		case SWBD117:
		case SWBD118:
		case SWBD129:
		case SWBD130:
		case SWBD124:	/* Dragon2 */
		case SWBD125:   /* Dragon2T */
		case SWBD134:   /* Qilin */
		case SWBD149:   /* Kestrel3 */
		case SWBD150:
			break;

		default:
			break;
    }

	return 0;
}

int silkworm_get_usb_irqno(int irq)
{
    switch (silkworm.board) {
    	case SWBD71:	/* Tomahawk uses internal interrupt 21 */
		case SWBD66:	/* Spirit */
		case SWBD85:	/* Spike-BP */
		case SWBD83:	/* Spike CP */
		case SWBD87:	/* Kestrel2 */
		case SWBD92:	/* Gladius */
		case SWBD109:   /* Stinger */
		case SWBD117:   /* Pharos */
		case SWBD118:
		case SWBD129:   /* Bulova2 */
		case SWBD130:   /* Superhak3 */
		case SWBD124:	/* Dragon2 */
		case SWBD125:   /* Dragon2T */
		case SWBD134:   /* Qilin */
		case SWBD149:   /* Kestrel3 */
		case SWBD150:
      		//return (21);
			break;
		default:
			break;
    }

    return irq;
}

static int __init silkworm_standalone(char *str)
{
	if (strcasecmp(str, "yes") == 0) {
		printk(KERN_INFO "Silkworm: BladeStandalone mode is set\n");
		printk(KERN_INFO "Silkworm: PCI init host controller\n");
		silkworm_pci_standalone_scan = 1;
		if (silkworm.board == SWBD88) {
			europa_setup_hose();
		}
	} else {
		silkworm_pci_standalone_scan = 0;
	}

	return 1;

}

__setup("BladeStandalone=", silkworm_standalone);

EXPORT_SYMBOL(silkworm_get_usb_irqno);
EXPORT_SYMBOL(silkworm_mask_usb_intr);
EXPORT_SYMBOL(silkworm_unmask_usb_intr);

