/*
 *
 * arch/ppc/platform/85xx/silkworm8548.c
 * Based on :
 * arch/ppc/platform/85xx/mpc85xx_cds_common.c
 *
 * MPC85xx CDS board specific routines
 *
 * Maintainer: Kumar Gala <kumar.gala@freescale.com>
 *
 * Copyright 2004 Freescale Semiconductor, Inc
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

/*
 * Brocade silkworm platform using Freescale 8548 CPU
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/major.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/ide.h>
#include <linux/seq_file.h>
#include <linux/serial.h>
#include <linux/module.h>
#include <linux/root_dev.h>
#include <linux/initrd.h>
#include <linux/tty.h>
#include <linux/serial_core.h>
#include <linux/fsl_devices.h>
#include <linux/ethtool.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/atomic.h>
#include <asm/time.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/open_pic.h>
#include <asm/i8259.h>
#include <asm/bootinfo.h>
#include <asm/pci-bridge.h>
#include <asm/mpc85xx.h>
#include <asm/irq.h>
#include <asm/immap_85xx.h>
#include <asm/cpm2.h>
#include <asm/ppc_sys.h>
#include <asm/kgdb.h>

#include <mm/mmu_decl.h>
#include <syslib/cpm2_pic.h>
#include <syslib/ppc85xx_common.h>
#include <syslib/ppc85xx_setup.h>

#include <brocade/platform.h>
#include <brocade/silkworm.h>
#include <brocade/silkworm_features.h>
#include <platforms/4xx/silkworm_cpld.h>
#include <brocade/cpld.h>
#include <brocade/silkworm_hwsem.h>
#include <brocade/sin.h>

extern sw_bd silkworm;
extern int cpld_init(void);
extern int silkworm_hwsem_init(int);
extern int  mpc85xx_pci1_last_busno;
extern void silkworm_openpic_msi_init();

/* For wacthdog */
#ifdef CONFIG_BOOKE_WDT
extern void ppc_booke_sw_wdt_heartbeat(void);
extern int sw_wdt_enable;
extern unsigned long sw_wdt_period;
extern u32 booke_wdt_enabled;
#endif

/* These are default values as in eval board */
unsigned long mpc85xx_pci1_lower_mem = 0x80000000;   
unsigned long mpc85xx_pci1_upper_mem = 0x9fffffff;
unsigned long mpc85xx_pci1_mem_offset = 0x00000000;

int mpc85xx_pci1hose_first_busno = 0x0;
int mpc85xx_pci1hose_bus_offset = 0x0;
int mpc85xx_pci1hose_last_busno = 0xff;

/* These defaults are from Elara board */
unsigned long mpc85xx_pex_lower_mem = 0xe9000000;
unsigned long mpc85xx_pex_upper_mem = 0xefffffff;
unsigned long mpc85xx_pex_mem_offset = 0x00000000;

/* These defaults are from Elara board */
int mpc85xx_pex1hose_first_busno = 0xf1;
int mpc85xx_pex1hose_bus_offset = 0xf1;
int mpc85xx_pex1hose_last_busno = 0xff;


/* Indicate that we want to do pci scan: Yes=1, No=0 */
int silkworm_pci_standalone_scan = 0;

static void silkworm85xx_setup_pci_phy_base(void);

/* platform speific warm reset flag which is exported to blade driver */
unsigned short silkworm_cpld_warm_reset_flag;

#ifndef CONFIG_PCI
unsigned long isa_io_base = 0;
unsigned long isa_mem_base = 0;
#endif

extern unsigned long total_memory;      /* in mm/init */

unsigned char __res[sizeof (bd_t)];

#ifndef CONFIG_SILKWORM_COMMON
static int cds_pci_slot = 2;
#endif

/* Internal interrupts are all Level Sensitive, and Positive Polarity */
static u_char silkworm85xx_openpic_initsenses[] __initdata = {
	MPC85XX_INTERNAL_IRQ_SENSES,
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* External  0: PHY_1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* External  1: PHY_2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* External  2: FPGA, PHY_3 and PHY4 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* External  3: condor */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* External  4 - CompactFlash */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* External  5 - CompactFlash */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* External  6: 1250-0 to BP */
	0x0,						/* External  7: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),	/* External  8: 1250-1 to BP */
	0x0,						/* External  9: */
	0x0,						/* External 10: */
	0x0,						/* External 11: */
};

static u_char silkworm85xx_openpic_initsenses_naptune[] __initdata = {
	MPC85XX_INTERNAL_IRQ_SENSES,
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  0: PB state Change */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  1: PB Attn */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  2: OCP state Chng */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  3: SYS INT-1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  4: CP Attn */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  5: Core Attn */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  6: PB Power Fail */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  7: SYS INT-2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  8: Core state chng */
	0x0,					   /* Ext  9: Blade Int */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext 10: Compact Flash */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext 11: USB */
};

static u_char silkworm85xx_openpic_initsenses_mace[] __initdata = {
	MPC85XX_INTERNAL_IRQ_SENSES,
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  0: PB state Change */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  1: PB Attn */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  2: OCP state Chng */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  3: SYS INT-1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  4: CP Attn */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  5: Core Attn */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  6: PB Power Fail */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  7: SYS INT-2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  8: Core state chng */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  9: Compact Flash */
	0x0,					   /* Ext 10: Blade Int */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext 11: USB */
};

static u_char silkworm85xx_openpic_initsenses_lance[] __initdata = {
	MPC85XX_INTERNAL_IRQ_SENSES,
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  0: PB state Change */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  1: PB Attn */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  2: OCP state Chng */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  3: SYS INT-1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  4: CP Attn */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  5: Core Attn */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  6: PB Power Fail */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  7: SYS INT-2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  8: Core state chng */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  9: Compact Flash */
	0x0,					   /* Ext 10: Blade Int */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext 11: USB */
};

static u_char silkworm85xx_openpic_initsenses_striker[] __initdata = {
	MPC85XX_INTERNAL_IRQ_SENSES,
	0x0, /* Ext  0: PB state Change */
	0x0, /* Ext  1: PB Attn */
	(0x0), /* Ext  2: OCP state Chng */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  3: SYS INT-1 */
	(0x0), /* Ext  4: CP Attn */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  5: Core Attn */
	(0x0), /* Ext  6: PB Power Fail */
	(0x0), /* Ext  7: SYS INT-2 */
	(0x0), /* Ext  8: Core state chng */
	(0x0), /* Ext  9: Compact Flash */
	0x0,					   /* Ext 10: Blade Int */
	(0x0), /* Ext 11: USB */
};

static u_char silkworm85xx_openpic_initsenses_spear[] __initdata = {
        MPC85XX_INTERNAL_IRQ_SENSES,
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  0: PB state Change */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  1: PB Attn */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  2: OCP state Chng */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  3: SYS INT-1 */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  4: CP Attn */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  5: Core Attn */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  6: PB Power Fail */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  7: SYS INT-2 */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  8: Core state chng */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  9: Compact Flash */
        0x0,                                       /* Ext 10: Blade Int */
        (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext 11: USB */
};

static u_char silkworm85xx_openpic_initsenses_thor[] __initdata = {
    MPC85XX_INTERNAL_IRQ_SENSES,
    0x0,                                       /* Ext  0: Reserved */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  1: ASIC attention */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  2: SFP Power */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  3: SYS ATTN */
    0x0,                                       /* Ext  4: Reserved */
    0x0,                                       /* Ext  5: Reserved */
    0x0,                                       /* Ext  6: Reserved */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  7: I2C */
    0x0,                                       /* Ext  8: Reserved */
    0x0,                                       /* Ext  9: Reserved */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext 10: IDE Compact Flash */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Ext 11: USB */
};

static u_char silkworm85xx_openpic_initsenses_odin[] __initdata = {
    MPC85XX_INTERNAL_IRQ_SENSES,
    0x0,					/* Ext 0 Reserved */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* Ext  1: Condor3  */
    0x0,					/* Ext 2 Reserved */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), 	/* Ext  3: Voltage monitor etc.  */ 
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  4: PCIE  */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* Ext  5: USB */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* Ext  6: I2C Master controller */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* Ext 7 : CPU internal devices*/
    0x0,				        /* Ext 8 : Reserved */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* Ext 9: IDE Compact Flash */
    0x0,					/* Ext 10: Reserved */
    0x0,					/* Ext 11: Reserved */
};


static u_char silkworm85xx_openpic_initsenses_elara[] __initdata = {
	MPC85XX_INTERNAL_IRQ_SENSES,
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Ext  0: USB   */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Ext  1: PS, Fan */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  2: Reserved */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Ext  3: Zeus -> CPU */
	0x0,										/* Ext 4 */
	0x0,										/* Ext 5 */
	0x0,										/* Ext 6: */
	0x0,										/* Ext 7: */
	0x0,										/* Ext 8: */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),	/* Ext  9: CF */
	0x0,										/* Ext 10: */
	0x0,										/* Ext 11: */
};

static u_char silkworm85xx_openpic_initsenses_elara2e[] __initdata = {
    MPC85XX_INTERNAL_IRQ_SENSES,
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Ext  0: USB   */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Ext  1: PS, Fan */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE), /* Ext  2: Reserved */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE), /* Ext  3: GE PHY */
    0x0,                                        /* Ext 4 */
    (IRQ_SENSE_EDGE  | IRQ_POLARITY_POSITIVE), /* Ext  5: 405EXr GPIO24 */
    (IRQ_SENSE_EDGE  | IRQ_POLARITY_POSITIVE), /* Ext  6: 405EXr GPIO25 */
    (IRQ_SENSE_EDGE  | IRQ_POLARITY_POSITIVE), /* Ext  7: 405EXr GPIO27 */
    0x0,                                        /* Ext 8: */
    (IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),  /* Ext  9: CF */
    0x0,                                        /* Ext 10: */
    0x0,                                        /* Ext 11: */
};


/* ************************************************************************ */
int
silkworm85xx_show_cpuinfo(struct seq_file *m)
{
	uint pvid, svid, phid1;
	uint memsize = total_memory;
	bd_t *binfo = (bd_t *) __res;
	unsigned int freq;

	/* get the core frequency */
	freq = binfo->bi_intfreq;

	pvid = mfspr(SPRN_PVR);
	svid = mfspr(SPRN_SVR);

	seq_printf(m, "Vendor\t\t: Freescale Semiconductor\n");
	seq_printf(m, "clock\t\t: %dMHz\n", freq / 1000000);
	seq_printf(m, "PVR\t\t: 0x%x\n", pvid);
	seq_printf(m, "SVR\t\t: 0x%x\n", svid);

	/* Display cpu Pll setting */
	phid1 = mfspr(SPRN_HID1);
	seq_printf(m, "PLL setting\t: 0x%x\n", ((phid1 >> 24) & 0x3f));

	/* Display the amount of memory */
	seq_printf(m, "Memory\t\t: %d MB\n", memsize / (1024 * 1024));

	return 0;
}

void __init
silkworm85xx_init_IRQ(void)
{
	bd_t *binfo = (bd_t *) __res;
	int i;

	if(silkworm.board == SWBD68) {
	    for(i = 0; i < 4; i++)
    	   silkworm85xx_openpic_initsenses_lance[i
            + MPC85xx_IRQ_EXT0 - MPC85xx_OPENPIC_IRQ_OFFSET]
                               = IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE;
	}


	/* Determine the Physical Address of the OpenPIC regs */
	phys_addr_t OpenPIC_PAddr = binfo->bi_immr_base +
						MPC85xx_OPENPIC_OFFSET;

	OpenPIC_Addr = ioremap(OpenPIC_PAddr, MPC85xx_OPENPIC_SIZE);

	switch (silkworm.board) {
	case SWBD55:
	OpenPIC_InitSenses = silkworm85xx_openpic_initsenses;
		OpenPIC_NumInitSenses = sizeof (silkworm85xx_openpic_initsenses);
		break;
	case SWBD64:
		OpenPIC_InitSenses = silkworm85xx_openpic_initsenses_thor;
		OpenPIC_NumInitSenses = sizeof (silkworm85xx_openpic_initsenses_thor);
		break;
	case SWBD133:
		OpenPIC_InitSenses = silkworm85xx_openpic_initsenses_odin;
        OpenPIC_NumInitSenses = sizeof (silkworm85xx_openpic_initsenses_odin);
		break;
	case SWBD62:
	case SWBD63:
	case SWBD77:
		OpenPIC_InitSenses = silkworm85xx_openpic_initsenses_naptune;
		OpenPIC_NumInitSenses =
			sizeof (silkworm85xx_openpic_initsenses_naptune);
		break;
	case SWBD67:  /* MACE */
		OpenPIC_InitSenses = silkworm85xx_openpic_initsenses_mace;
		OpenPIC_NumInitSenses =
			sizeof (silkworm85xx_openpic_initsenses_mace);
		break;
	case SWBD68: /* Lance */
		OpenPIC_InitSenses = silkworm85xx_openpic_initsenses_lance;
		OpenPIC_NumInitSenses =
			sizeof (silkworm85xx_openpic_initsenses_lance);
		break;

	case SWBD76:  /* ELARA */
		OpenPIC_InitSenses = silkworm85xx_openpic_initsenses_elara;
		OpenPIC_NumInitSenses =
			sizeof (silkworm85xx_openpic_initsenses_elara);
			/* Add the Message registers setup too */
			openpic_set_sources(60, 4, OpenPIC_Addr + 0x11600);
		break;

	case SWBD79: /* Spear CP */
		OpenPIC_InitSenses = silkworm85xx_openpic_initsenses_spear;
		OpenPIC_NumInitSenses =
			sizeof (silkworm85xx_openpic_initsenses_spear);
		break;

	case SWBD89: /* Striker */
		OpenPIC_InitSenses = silkworm85xx_openpic_initsenses_striker;
		OpenPIC_NumInitSenses =
			sizeof (silkworm85xx_openpic_initsenses_striker);
		break;
	}

	/* Skip reserved space and internal sources */
#ifdef CONFIG_MPC8548
	openpic_set_sources(0, 48, OpenPIC_Addr + 0x10200);
#else
	openpic_set_sources(0, 32, OpenPIC_Addr + 0x10200);
#endif
	/* Map PIC IRQs 0-11 */
	openpic_set_sources(48, 12, OpenPIC_Addr + 0x10000);

	/* we let openpic interrupts starting from an offset, to
	 * leave space for cascading interrupts underneath.
	 */
	openpic_init(MPC85xx_OPENPIC_IRQ_OFFSET);

	/* This is for MSI device interrupts */
	if (silkworm.board == SWBD76) {
		extern void silkworm_openpic_msi_init();
		/* Enable the register interrupts */
		silkworm_openpic_msi_init();
	}

#ifdef CONFIG_PCI
/*
	openpic_hookup_cascade(PIRQ0A, "82c59 cascade", i8259_irq);

	for (i = 0; i < NUM_8259_INTERRUPTS; i++)
		irq_desc[i].handler = &i8259_pic;

	i8259_init(0);
*/
#endif

	return;
}

#ifdef CONFIG_PCI
/*
 * interrupt routing
 */
int
mpc85xx_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
#ifdef CONFIG_SILKWORM_COMMON
	/* Boot PROM configures the IRQ for each PCI device.  Linux scans the device and
	 * just uses what the bootrom set.
	 */
	int irq;

	if(dev)
		irq = dev->irq;
	else
		irq = -1;
	return irq;
#else
	struct pci_controller *hose = pci_bus_to_hose(dev->bus->number);

	if (!hose->index)
	{
		/* Handle PCI1 interrupts */
		char pci_irq_table[][4] =
			/*
			 *      PCI IDSEL/INTPIN->INTLINE
			 *        A      B      C      D
			 */

			/* Note IRQ assignment for slots is based on which slot the elysium is
			 * in -- in this setup elysium is in slot #2 (this PIRQA as first
			 * interrupt on slot */
		{
			{ 0, 1, 2, 3 }, /* 16 - PMC */
			{ 0, 1, 2, 3 }, /* 17 P2P (Tsi320) */
			{ 0, 1, 2, 3 }, /* 18 - Slot 1 */
			{ 1, 2, 3, 0 }, /* 19 - Slot 2 */
			{ 2, 3, 0, 1 }, /* 20 - Slot 3 */
			{ 3, 0, 1, 2 }, /* 21 - Slot 4 */
		};

		const long min_idsel = 16, max_idsel = 21, irqs_per_slot = 4;
		int i, j;

		for (i = 0; i < 6; i++)
			for (j = 0; j < 4; j++)
				pci_irq_table[i][j] =
					((pci_irq_table[i][j] + 5 -
					  cds_pci_slot) & 0x3) + PIRQ0A;

		return PCI_IRQ_TABLE_LOOKUP;
	} else {
		/* Handle PCI2 interrupts (if we have one) */
		char pci_irq_table[][4] =
		{
			/*
			 * We only have one slot and one interrupt
			 * going to PIRQA - PIRQD */
			{ PIRQ1A, PIRQ1A, PIRQ1A, PIRQ1A }, /* 21 - slot 0 */
		};

		const long min_idsel = 21, max_idsel = 21, irqs_per_slot = 4;

		return PCI_IRQ_TABLE_LOOKUP;
	}
#endif /* SILKWORM */
}

extern int exclude_pex;
int
mpc85xx_exclude_device(u_char bus, u_char devfn)
{
    int rc = PCIBIOS_SUCCESSFUL;

    switch (silkworm.board) {
	case SWBD76: /* ELARA */
	case SWBD133: /* ODIN */
		/* Don't exclude 8548 host bridge on Scimitar platforms */
		return PCIBIOS_SUCCESSFUL;
	case SWBD55:
	case SWBD67: 		/* MACE */
	case SWBD68: 		/* LANCE */
    case SWBD79:	/* Spear CP */
    case SWBD89:	/* Striker */
        break;
    case SWBD62:        /* Fall thru */
    case SWBD63:        /* Fall thru */
    case SWBD64:        /* Thor */
    case SWBD77:        /* Neptune */
    default:
        /* Skip the host bridge */
        if (bus == 0 && PCI_SLOT(devfn) == 0) {
            rc = PCIBIOS_DEVICE_NOT_FOUND;
			printk("PCI %s: EXCLUDE bus 0x%x slot 0x%x\n",
				   __FUNCTION__, bus, PCI_SLOT(devfn));
		}
    }
    return rc;
}

#endif /* CONFIG_PCI */


/* ************************************************************************
 *
 * Setup the ethernet 
 *
 */

static void __init
silkworm85xx_setup_enet(void)
{
	bd_t *binfo = (bd_t *) __res;
	struct gianfar_platform_data *pdata;
	struct platform_device swap_ppc_platform_device;
	
	switch (silkworm.board) {
    case SWBD64:	/* Thor */
		/*
		 * defect 248525, Thor's management interface is connected to
		 * a BCM5241, not a 5461 as originally documented below.
		 * that means the PHY doesn't support GigE even though the
		 * MAC does.
		 */
		/*
		 * eTSEC1---+---- (GMII) --> Phy-BCM5241 --> RJ45 (10/100)
		 *
		 * eTSEC2---+
		 *          |
		 * eTSEC3---+---- UNDEFINED (NONE)
		 *          |
		 * eTSEC4---+
		 */

        pdata = (struct gianfar_platform_data *)
                ppc_sys_get_pdata(MPC85xx_eTSEC1);
        if (pdata) {
            pdata->board_flags = FSL_GIANFAR_BRD_COPPER_FORCED;
			/* 248525, toggle off GigE support bit */
			pdata->device_flags &= ~(FSL_GIANFAR_DEV_HAS_GIGABIT);
            pdata->interruptPHY = MPC85xx_IRQ_EXT0;
            pdata->phyid = 1;
            /* fixup phy address */
            pdata->phy_reg_addr += binfo->bi_immr_base;
            memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
        }
		/* Remove the emac2, 3, and 4 as they are not connected */
		ppc_sys_device_remove(MPC85xx_eTSEC2);
		ppc_sys_device_remove(MPC85xx_eTSEC3);
		ppc_sys_device_remove(MPC85xx_eTSEC4);
        break;

	case SWBD55:
		/*
		 * eTSEC1---+
		 *          |---- (GMII) --> Phy-BCM5461 --> RJ45 (10/100/1000)
		 * eTSEC2---+
		 *
		 * eTSEC3---+
		 *          |---- (RMII) --> Phy-BCM5221 --> Backplane (10/100)
		 * eTSEC4---+
		 */

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC1);
		if (pdata) {
			pdata->board_flags = FSL_GIANFAR_BRD_COPPER_FORCED | FSL_GIANFAR_BRD_IS_NOT_TRUSTED;
			pdata->interruptPHY = MPC85xx_IRQ_EXT0;
			pdata->phyid = 1;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC2);
		if (pdata) {
			pdata->board_flags = FSL_GIANFAR_BRD_COPPER_FORCED | FSL_GIANFAR_BRD_IS_NOT_TRUSTED;
			pdata->interruptPHY = MPC85xx_IRQ_EXT1;
			pdata->phyid = 2;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 1;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC3);
		if (pdata) {
			pdata->board_flags = FSL_GIANFAR_BRD_IS_REDUCED;
			pdata->interruptPHY = MPC85xx_IRQ_EXT3;
			pdata->phyid = 3;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 2;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_100baseT_Full;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC4);
		if (pdata) {
			pdata->board_flags = FSL_GIANFAR_BRD_IS_REDUCED;
			pdata->interruptPHY = MPC85xx_IRQ_EXT3;
			pdata->phyid = 4;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 3;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_100baseT_Full;
		}
		break;

	case SWBD62:
	case SWBD77:

		/*
		 * eTSEC1----- (RGMII) ---> NO-Phy ----> Ethernet-Switch GigE
		 *         
		 * eTSEC2--- (RGMII) --> Phy-BCM5461S (0x1B) --->OCP Serdes GigE
		 *
		 * eTSEC3--- (RGMII) --> Phy-BCM5461 (0x1C) -->RJ45(10/100/1000) 
		 *
		 * eTSEC4--- (RGMII) --> Phy-BCM5461 (0x1D) -->RJ45(10/100/1000)
		 */

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC1);
		if (pdata) {
			pdata->board_flags = 0;
			/* No phy */
			pdata->phyid = FSL_GIANFAR_BRD_HAS_NO_PHY;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 2;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_1000baseT_Full;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC2);
		if (pdata) {
			pdata->board_flags = 0;
			pdata->phyid = 0x1B;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 1;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_1000baseT_Full;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC3);
		if (pdata) {
			pdata->board_flags = FSL_GIANFAR_BRD_IS_NOT_TRUSTED;
			pdata->phyid = 0x1C;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC4);
		if (pdata) {
			pdata->board_flags = FSL_GIANFAR_BRD_IS_NOT_TRUSTED;
			pdata->phyid = 0x1D;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 3;
		}

		/*
		 * In order to keep the rest of software stack unchanged
		 * swap the platform device table so that:         
		 *         
		 * eTSEC1 ---> will become eTSEC3 (eth2) 
		 *         
		 * eTSEC2 ---> no swap (eth1)
		 *
		 * eTSEC3 ---> will become eTSEC1 (eth0)
		 *
		 * eTSEC4 ---> eth3 on zentron
		 */

		/* Replace eTsec1 with eTsec3 */
		memcpy(&swap_ppc_platform_device,
			&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
					sizeof(struct platform_device));
		memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
			&(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
					sizeof(struct platform_device));
		memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
			&swap_ppc_platform_device,
					sizeof(struct platform_device));

		/* Use the PCA9665 if asked for */
		if (strstr(cmd_line, "iic_pca")) {
			ppc_sys_device_remove(MPC85xx_IIC2);
			printk(KERN_INFO "SILKWORM: PCA9665 I2C adapter is primary now\n");
		} 
		break;

	case SWBD63:

		/*
		 * eTSEC1----- (RGMII) ---> NO-Phy ----> Ethernet-Switch GigE
		 *         
		 * eTSEC2--- (RGMII) --> Phy-BCM5461S (0xB) ---> OCP Serdes GigE
		 *
		 * eTSEC3--- (RGMII) --> Phy-BCM5461 (0x9) --> RJ45(10/100/1000) 
		 *
		 * eTSEC4--- NONE
		 */

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC1);
		if (pdata) {
			pdata->board_flags = 0;
			/* No phy */
			pdata->phyid = FSL_GIANFAR_BRD_HAS_NO_PHY;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 3;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_1000baseT_Full;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC2);
		if (pdata) {
			pdata->board_flags = 0;
			pdata->phyid = 0xB;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 2;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_1000baseT_Full;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC3);
		if (pdata) {
			pdata->board_flags = 0;
			pdata->phyid = 0x1;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
		}

		/*
		 * In order to keep the rest of software stack unchanged
		 * swap the platform device table so that:         
		 *         
		 * eTSEC1 ---> will become eTSEC3 (eth2) 
		 *         
		 * eTSEC2 ---> no swap (eth1)
		 *
		 * eTSEC3 ---> will become eTSEC1 (eth0)
		 *
		 * eTSEC4 ---> not connected
		 */

		/* Replace eTsec1 with eTsec3 */
		memcpy(&swap_ppc_platform_device,
			&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
					sizeof(struct platform_device));
		memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
			&(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
					sizeof(struct platform_device));
		memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
			&swap_ppc_platform_device,
					sizeof(struct platform_device));

		/* Remove the emac4 as it is not connected */
		ppc_sys_device_remove(MPC85xx_eTSEC4);

		break;

	case SWBD67: /* MACE */
		/*
		 * eTSEC1--- (RGMII) -->Phy-Marvel(88E1116)(0x1) --> BE
		 *         
		 * eTSEC2--- (RGMII) -->NO-Phy
		 *
		 * eTSEC3--- (RGMII) -->Phy-Marvel(88E1116)(0x3) -->RJ45(10/100/1000) 
		 *
		 * eTSEC4--- (RGMII) -->NO-Phy
		 */

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC1);
		if (pdata) {
			pdata->board_flags = 0;
			
			pdata->phyid = 0x1;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 3;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_1000baseT_Full;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC2);
		if (pdata) {
			pdata->board_flags = 0;
			pdata->phyid = FSL_GIANFAR_BRD_HAS_NO_PHY;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 2;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_1000baseT_Full;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC3);
		if (pdata) {
			pdata->board_flags = FSL_GIANFAR_BRD_IS_NOT_TRUSTED;
			pdata->phyid = 0x3;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC4);
		if (pdata) {
			pdata->board_flags = 0;
			pdata->phyid = FSL_GIANFAR_BRD_HAS_NO_PHY;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 1;
		}

		/*
		 * In order to keep the rest of software stack unchanged
		 * swap the platform device table so that:         
		 *         
		 * eTSEC1 ---> will become eTSEC3 (eth2) 
		 *         
		 * eTSEC2 ---> no swap (eth1)
		 *
		 * eTSEC3 ---> will become eTSEC1 (eth0)
		 *
		 * eTSEC4 ---> no swap (eth3) remove in production
		 */

		/* Replace eTsec1 with eTsec3 */
		memcpy(&swap_ppc_platform_device,
			&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
					sizeof(struct platform_device));
		memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
			&(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
					sizeof(struct platform_device));
		memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
			&swap_ppc_platform_device,
					sizeof(struct platform_device));

		/* TODO */
		/* In the production build disable the etsec4 - service port */
		/* ppc_sys_device_remove(MPC85xx_eTSEC4); */

		break;

	case SWBD68: /* Lance */
		/*
		 * eTSEC1--- (RGMII) -->Phy-Marvel(88E1116)(0x1) --> BE
		 *         
		 * eTSEC2--- (RGMII) --> NO-Phy --> SP 
		 *
		 * eTSEC3--- (RGMII) -->Phy-Marvel(88E1116)(0x2) -->RJ45(10/100/1000) 
		 *
		 * eTSEC4--- (RGMII) -->Phy-Marvel(88E1116)(0x3) -->RJ45(10/100/1000)
		 */

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC1);
		if (pdata) {
			pdata->board_flags = 0;
			
			pdata->phyid = 0x1;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 3;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_100baseT_Full;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC2);
		if (pdata) {
			pdata->board_flags = 0;
			/* No phy */
			pdata->phyid = FSL_GIANFAR_BRD_HAS_NO_PHY;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 2;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_100baseT_Full;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC3);
		if (pdata) {
			pdata->board_flags = FSL_GIANFAR_BRD_IS_NOT_TRUSTED;
			pdata->phyid = 0x10;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC4);
		if (pdata) {
			pdata->board_flags = FSL_GIANFAR_BRD_IS_NOT_TRUSTED;
			pdata->phyid = 0x11;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enet1addr, 6);
		}

		/*
		 * In order to keep the rest of software stack unchanged
		 * swap the platform device table so that:         
		 *         
		 * eTSEC1 ---> will become eTSEC3 (eth2) 
		 *         
		 * eTSEC2 ---> no swap (eth1)
		 *
		 * eTSEC3 ---> will become eTSEC1 (eth0)
		 *
		 * eTSEC4 ---> no swap (eth3) remove in production
		 */

		/* Replace eTsec1 with eTsec3 */
		memcpy(&swap_ppc_platform_device,
			&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
					sizeof(struct platform_device));
		memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
			&(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
					sizeof(struct platform_device));
		memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
			&swap_ppc_platform_device,
					sizeof(struct platform_device));

		/* TODO */
		/* In the production build disable the etsec4 - service port */
		/* ppc_sys_device_remove(MPC85xx_eTSEC4); */

		break;

        case SWBD79: /* SPEAR CP */
                /*
                 * eTSEC1--- (MII) -->NO-Phy->ESWITCH(10/100)
                 *
                 * eTSEC2--- (RGMII) -->NO-Phy NO CONNECT
                 *
                 * eTSEC3--- (RGMII) -->Phy-BCM5461(0x1c) -->RJ45(10/100/1000)
                 *
                 * eTSEC4--- (RGMII) -->NO-Phy NO CONNECT
                 */

                pdata = (struct gianfar_platform_data *)
                                ppc_sys_get_pdata(MPC85xx_eTSEC1);
                if (pdata) {
                        pdata->board_flags = 0;

                        pdata->phyid = FSL_GIANFAR_BRD_HAS_NO_PHY;
                        // The following seems to work for now
                        pdata->phyid = 0x1;
                        /* fixup phy address */
                        pdata->phy_reg_addr += binfo->bi_immr_base;
                        memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
                        pdata->mac_addr[5] += 3;
                        /* Force the link speed, no autoneg */
                        pdata->phy_flags = SUPPORTED_1000baseT_Full;
                }

                pdata = (struct gianfar_platform_data *)
                                ppc_sys_get_pdata(MPC85xx_eTSEC2);
                if (pdata) {
                        pdata->board_flags = 0;
                        pdata->phyid = FSL_GIANFAR_BRD_HAS_NO_PHY;
                        /* fixup phy address */
                        pdata->phy_reg_addr += binfo->bi_immr_base;
                        memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
                        pdata->mac_addr[5] += 2;
                        /* Force the link speed, no autoneg */
                        pdata->phy_flags = SUPPORTED_1000baseT_Full;
                }

                pdata = (struct gianfar_platform_data *)
                                ppc_sys_get_pdata(MPC85xx_eTSEC3);
                if (pdata) {
                        pdata->board_flags = FSL_GIANFAR_BRD_IS_NOT_TRUSTED;
                        pdata->phyid = 0x1c;
                        /* fixup phy address */
                        pdata->phy_reg_addr += binfo->bi_immr_base;
                        memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
                }

                pdata = (struct gianfar_platform_data *)
                                ppc_sys_get_pdata(MPC85xx_eTSEC4);
                if (pdata) {
                        pdata->board_flags = 0;
                        pdata->phyid = FSL_GIANFAR_BRD_HAS_NO_PHY;
                        /* fixup phy address */
                        pdata->phy_reg_addr += binfo->bi_immr_base;
                        memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
                        pdata->mac_addr[5] += 1;
                }

                /*
                 * In order to keep the rest of software stack unchanged
                 * swap the platform device table so that:
                 *
                 * eTSEC1 ---> will become eTSEC3 (eth2)
                 *
                 * eTSEC2 ---> no swap (eth1)
                 *
                 * eTSEC3 ---> will become eTSEC1 (eth0)
                 *
                 * eTSEC4 ---> no swap (eth3) remove in production
                 */

                /* Replace eTsec1 with eTsec3 */
                memcpy(&swap_ppc_platform_device,
                        &(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
                                        sizeof(struct platform_device));
                memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
                        &(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
                                        sizeof(struct platform_device));
                memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
                        &swap_ppc_platform_device,
                                        sizeof(struct platform_device));

                /* TODO */
                /* In the production build disable the etsec4 - service port */
                /* ppc_sys_device_remove(MPC85xx_eTSEC4); */

                break;

	case SWBD89: /* Striker */
		/*
		 * eTSEC1 ---> will become eTSEC3 (eth2) 
		 *         
		 * eTSEC2 ---> no swap (eth1)
		 *
		 * eTSEC3 ---> will become eTSEC1 (eth0)
		 */

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC1);
		if (pdata) {
			pdata->board_flags = 0;
			
			/* No phy */
			pdata->phyid = FSL_GIANFAR_BRD_HAS_NO_PHY;

			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 1;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_1000baseT_Full;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC2);
		if (pdata) {
			pdata->board_flags = 0;
			/* No phy */
			pdata->phyid = FSL_GIANFAR_BRD_HAS_NO_PHY;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			pdata->mac_addr[5] += 2;
			/* Force the link speed, no autoneg */
			pdata->phy_flags = SUPPORTED_100baseT_Full;
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC3);
		if (pdata) {
			pdata->board_flags = FSL_GIANFAR_BRD_IS_NOT_TRUSTED;
			pdata->phyid = 0x10;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
		}

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC4);
		if (pdata) {
			pdata->board_flags = FSL_GIANFAR_BRD_IS_NOT_TRUSTED;
			pdata->phyid = 0x11;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enet1addr, 6);
		}

		/*
		 * In order to keep the rest of software stack unchanged
		 * swap the platform device table so that:         
		 *         
		 * eTSEC1 ---> will become eTSEC3 (eth2) 
		 *         
		 * eTSEC2 ---> no swap (eth1)
		 *
		 * eTSEC3 ---> will become eTSEC1 (eth0)
		 *
		 * eTSEC4 ---> no swap (eth3) remove in production
		 */

		/* Replace eTsec1 with eTsec3 */
		memcpy(&swap_ppc_platform_device,
			&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
					sizeof(struct platform_device));
		memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
			&(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
					sizeof(struct platform_device));
		memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC3]),
			&swap_ppc_platform_device,
					sizeof(struct platform_device));

		/* TODO */
		/* In the production build disable the etsec4 - service port */
		//ppc_sys_device_remove(MPC85xx_eTSEC1); 
		ppc_sys_device_remove(MPC85xx_eTSEC2); 
		//ppc_sys_device_remove(MPC85xx_eTSEC3); 
		ppc_sys_device_remove(MPC85xx_eTSEC4); 

		break;


	case SWBD76: /* ELARA */
		/*
		 * eTSEC1--- (GMII) -->Phy-Broadcom BCM5461(0x1) -->RJ45
		 *         
		 * eTSEC2--- NC
		 *
		 * eTSEC3--- NC
		 *
		 * eTSEC4--- NC
		 */

		pdata = (struct gianfar_platform_data *)
				ppc_sys_get_pdata(MPC85xx_eTSEC1);
		if (pdata) {
			pdata->board_flags = 0;
			
			pdata->phyid = 0x1;
			/* fixup phy address */
			pdata->phy_reg_addr += binfo->bi_immr_base;
			memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
			/* Force the link speed, no autoneg */
			pdata->phy_flags = 0;
		}

		ppc_sys_device_remove(MPC85xx_eTSEC2);
		ppc_sys_device_remove(MPC85xx_eTSEC3);
		ppc_sys_device_remove(MPC85xx_eTSEC4);

		break;
	case SWBD133: /* Odin */
        /*
		 * eTSEC2--- NC
	     *		
         * eTSEC2--- (RGMII) -->Phy-Broadcom BCM54610(0x1) -->RJ45
         *         
         *
         * eTSEC3--- NC
         *
         * eTSEC4--- NC
         */

        pdata = (struct gianfar_platform_data *)
                ppc_sys_get_pdata(MPC85xx_eTSEC2);
        if (pdata) {
            pdata->board_flags = 0;

            pdata->phyid = 0x1;
            /* fixup phy address */
            pdata->phy_reg_addr += binfo->bi_immr_base;
            memcpy(pdata->mac_addr, binfo->bi_enetaddr, 6);
            /* Force the link speed, no autoneg */
            pdata->phy_flags = 0;
        }

		memcpy(&(ppc_sys_platform_devices[MPC85xx_eTSEC1]),
            &(ppc_sys_platform_devices[MPC85xx_eTSEC2]),
                    sizeof(struct platform_device));

        ppc_sys_device_remove(MPC85xx_eTSEC2);
        ppc_sys_device_remove(MPC85xx_eTSEC3);
        ppc_sys_device_remove(MPC85xx_eTSEC4);

        break;


	default:
		/* Error - Unknown Silkworm platform */
		break;
	}

	ppc_sys_device_remove(MPC85xx_TSEC1);
	ppc_sys_device_remove(MPC85xx_TSEC2);
	ppc_sys_device_remove(MPC85xx_FEC);

}

/* The PCI mappings are BD specific */
static void silkworm85xx_setup_pci_phy_base()
{
	switch (silkworm.board) {

	case SWBD62:
	case SWBD77:
		/* @kevin
		 * For Zentron without Co-CPU, 
		 * PCI1 memory includes the blades and the USB
		 * controller. This encompasses 0x8000_0000 ... 0x0xe9ffffff
		 *
		 * There can be up to 256 busses (0-255). Allocate
		 * first 254 busses to PCI1 controller
		 */
		mpc85xx_pci1_lower_mem = 0x80000000;   
		mpc85xx_pci1_upper_mem = 0xe9ffffff;
		mpc85xx_pci1_mem_offset = 0x00000000;

		mpc85xx_pci1hose_first_busno = 0x0;
		mpc85xx_pci1hose_last_busno = 0xfd;
		mpc85xx_pci1hose_bus_offset = 0x0;
		
		break;
	case SWBD67:
	case SWBD79: /* Spear CP */
		/*
		 * Do not get confused by the values setup by uboot
		 * The POW value of lower mem specify what secondary
		 * side is going to see. These values are for primary
		 * side. But the BAR reported in the config space are
		 * for secondary side. We apply right offset to fix them.
		 * Be careful for the DMA though.
		 * CPU generates 8000_0000 --POW--> 4000_0000
		 * BAR is setup as 4000_0000
		 * Linux uses HOSE offset and makes BAR usable from CPU side
		 */

		/*
		 * For Zentron, PCI1 memory includes the blades and the USB
		 * controller. This encompasses 0x8000_0000 ... 0x0xe9ffffff
		 *
		 * There can be up to 256 busses (0-255). Allocate
		 * first 254 busses to PCI1 controller
		 */

		mpc85xx_pci1_lower_mem = 0x80000000;   
		mpc85xx_pci1_upper_mem = 0xe9ffffff;
		mpc85xx_pci1_mem_offset = 0x00000000;

		mpc85xx_pci1hose_first_busno = 0x0;
		mpc85xx_pci1hose_last_busno = 0xfd;
		mpc85xx_pci1hose_bus_offset = 0x0;

		/*
		 * PCIe 64MB window, start on 64MB boundary
		 * Allocate last 2 busses to PCIe
		 */

		mpc85xx_pex_lower_mem = 0xeC000000;
		mpc85xx_pex_upper_mem = 0xefffffff;
		mpc85xx_pex_mem_offset = 0x00000000;

		mpc85xx_pex1hose_first_busno = 0xfe;
		mpc85xx_pex1hose_last_busno = 0xff;
		mpc85xx_pex1hose_bus_offset = 0x0;
		break;

	case SWBD64:
	case SWBD89:
		mpc85xx_pci1_lower_mem = 0x80000000;   
		mpc85xx_pci1_upper_mem = 0xefffffff;
		mpc85xx_pci1_mem_offset = 0x00000000;
        break;

	case SWBD68:
		/*
		 * Currently supporting only 1GB RAM configuration.
		 * The configuration with in the #if support 2GB RAM configuraiton 
		 * but there are issues with memory mapping of Condor registers.
		 * TBD: Review these values and verify condor and USB are working fine.
		 */
		
		mpc85xx_pci1_lower_mem = 0x40000000;   
		mpc85xx_pci1_upper_mem = 0xe8efffff;
		mpc85xx_pci1_mem_offset = 0x00000000;

		mpc85xx_pex_lower_mem = 0xe9000000;   
		mpc85xx_pex_upper_mem = 0xefffffff;
		mpc85xx_pex_mem_offset = 0x00000000;
	
		break;

	case SWBD55:

		{
			sin_rec_t sr;
			int rc, index = 0;

			rc = sin_find_sr(SIN_CLASS_SYSTEM, SIN_CLASS_SYSTEM_PCI_BASE,
						&sr, &index);

			if (rc == 0) {
				/*
				 * Found a record that defines the PCI starting
				 * base address for the system this blade is in.
				 */
				mpc85xx_pci1_lower_mem = ((u_int *)(sr.sr_data))[0];
				printk("%s: PCI BASE ADDDRESs 0x%8.8x 0x%8.8lx\n",
					   __FUNCTION__, ((u_int *)(sr.sr_data))[0],
					   mpc85xx_pci1_lower_mem);
			} else {

				/*
				 * If there is no record, then we must be running
				 * in a system with older software. That could
				 * be a Saturn, so set the lower mem value to what
				 * was used in Saturn with FOS 5.3.0
				 */
				mpc85xx_pci1_lower_mem = 0x40000000;
				printk("%s: PCI BASE ADDRESS RECORD NOT FOUND\n",
					   __FUNCTION__);
			}

			mpc85xx_pci1_upper_mem = 0xefffffff;
			mpc85xx_pci1_mem_offset = 0x00000000;

		}
		break;
	case SWBD63:
		/*
		 * Nothing to setup for Zentron co-CPU PCI/PCIe. PCI bus
		 * is not used. PCIe is used as an endpoint (no config scans)
		 * and the hardware is setup by the bootloader. The default
		 * defintions are defined in:
		 *
		 *    MPC85XX_PEX_COCPU_LOWER_MEM
		 *    MPC85XX_PEX_COCPU_UPPER_MEM
		 *    MPC85XX_PEX_COCPU_MEM_OFFSET
		 *
		 */
		break;

	case SWBD76: /* ELARA */
		mpc85xx_pci1_lower_mem = 0x80000000;   
		mpc85xx_pci1_upper_mem = 0xafffffff;
		mpc85xx_pci1_mem_offset = 0x00000000;

		mpc85xx_pex_lower_mem = 0xe0000000;
		mpc85xx_pex_upper_mem = 0xefffffff;
		//silkworm_pci_standalone_scan = 1;
		break;

	case SWBD133: /* ODIN */

		printk("Odin PCIE setup \n");
		mpc85xx_pci1_lower_mem = 0x80000000;   
		mpc85xx_pci1_upper_mem = 0xdfffffff;
		mpc85xx_pci1_mem_offset = 0x00000000;

		mpc85xx_pex_lower_mem = 0xe0000000;
		mpc85xx_pex_upper_mem = 0xefffffff;
		break;	
	default:
		printk(KERN_WARNING "PCI base not defined for the BD"
				"using default\n");
		break;
	}
}

/* ************************************************************************
 *
 * Setup the architecture
 *
 */
static void __init
silkworm85xx_setup_arch(void)
{
	bd_t *binfo = (bd_t *) __res;
	unsigned int freq;

	/* get the core frequency */
	freq = binfo->bi_intfreq;

	printk("silkworm85xx_setup_arch\n");

	/* Set loops_per_jiffy to a half-way reasonable value,
	   for use until calibrate_delay gets called. */
	loops_per_jiffy = freq / HZ;

#ifdef CONFIG_PCI
	/* XXX: may be delete following */
	/* VIA IDE configuration */
        //ppc_md.pcibios_fixup = silkworm85xx_pcibios_fixup;


	/* setup PCI host bridges */
	if (silkworm.board != SWBD63) {
		mpc85xx_setup_hose();
	}


#endif

#ifdef CONFIG_SERIAL_8250
	 mpc85xx_early_serial_map();
#endif

#ifdef CONFIG_SERIAL_TEXT_DEBUG
	/* Invalidate the entry we stole earlier the serial ports
	 * should be properly mapped */
	 invalidate_tlbcam_entry(num_tlbcam_entries - 1);
#endif

	/* setup the board related information for the enet controllers */
	silkworm85xx_setup_enet();
#ifdef  CONFIG_ROOT_NFS
		ROOT_DEV = Root_NFS;
#else
	ROOT_DEV = Root_HDA1;
#endif

	cpld_init();
        if ( ppc_md.progress ) ppc_md.progress("silkfsl: cpld done", 0x3eab);

	/* By default the USB interrupts are forced to be disabled */	
	switch (silkworm.board) {
	case SWBD62:
	case SWBD64:
	case SWBD79:
	case SWBD67:
	case SWBD68:
	case SWBD89:
	case SWBD76: /* ELARA */
	case SWBD77:
	case SWBD133:
		silkworm_mask_usb_intr();
		break;
	}

	silkworm_hwsem_init(silkworm.board);

	/* Identify the system */
	printk("Brocade Silkworm port (C) 2006 Brocade Communications Systems, Inc.\n");
}

#if defined(CONFIG_BRCD_CPLD_IDE)
static void
cpld_scimitarfsl_ide_init_hwif_ports(hw_regs_t * hw, unsigned long  data_port, unsigned long  ctrl_port, int *irq)
{
	unsigned long  *p;
	int i;
	static uint8_t *ata_base = NULL;

	p = hw->io_ports;
	*p = 0;

	if (irq)
		*irq = 0;
	
	/* IS this correct?  How else do we register only ide0 ? */
	if(ata_base)
		return;

	if (data_port != 0)
		return;

	/* Scimitar register definitions same as Pulsar */ 
	if ((ata_base = (void*) ioremap((unsigned long)PULSAR_CPLD_IDE_BASE_ADDR,
			(unsigned long)PULSAR_CPLD_IDE_SIZE *4)) == NULL) {
		printk(KERN_WARNING "cpld_ide_init_hwif_ports: failed ioremap "
				"of task file\n");
		return;
	}

	/* assign the hw ports
	* hack - the ide driver uses inb() to do the read - inb() 
	* adds _IO_BASE to the address before reading - need to subtract 
	* that off here, or get the ide driver
	* to use readb instead (but still work with PCI ide controllers...)
	* Also note that we shift the lower 16 bit address by 1
	*/

	for (i = 0; i <= IDE_STATUS_OFFSET; i++) {
		*p++ = (unsigned long) 
			 ata_base + ((PULSAR_REGISTER_FILE1_REG + i) << 1) - _IO_BASE;
#if 0
        printk("ATA ioport[%d] @0x%p (reg: 0x%x)\n",
               i,p,((PULSAR_REGISTER_FILE1_REG + i) << 1));
#endif
	}
	hw->io_ports[IDE_CONTROL_OFFSET] = 
		(unsigned long) ata_base + (PULSAR_REGISTER_FILE2_REG << 1) - _IO_BASE;
#if 0
    printk("ATA ioport[%d] @0x%p (control offset reg: 0x%x)\n",
            IDE_CONTROL_OFFSET,hw->io_ports[IDE_CONTROL_OFFSET],
            (PULSAR_REGISTER_FILE2_REG << 1));
#endif
	
	if (irq) {
		switch (silkworm.board) {
		case SWBD55:
			/* The Scimitar electrical specs 4/4/06 */
			*irq = MPC85xx_IRQ_EXT5;
			break;

		case SWBD64:        // Thor
		case SWBD63:
		case SWBD62:
		case SWBD77:
		//case SWBD133:
			/* The Zentron/Naptune CP specs 0.5 11/28/06 */
			*irq = MPC85xx_IRQ_EXT10;
			break;
		case SWBD67:
		case SWBD68:
		case SWBD76: /* ELARA */
		case SWBD133: /* Odin */
			/* The Mace CP+Jr specs 0.2 03/20/07 */
			*irq = MPC85xx_IRQ_EXT9;
			break;

		case SWBD89:
			*irq = MPC85xx_IRQ_EXT5;
			break;

                case SWBD79: /* SPEAR CP */
                        /* The Mace CP+Jr specs 0.2 03/20/07 */
                        *irq = MPC85xx_IRQ_EXT9;
                        break;
		}
	}

	ide_hwifs[data_port].noprobe = 0;

	printk(KERN_INFO "Silkworm CPLD ATA interface configured :"
					"16 bit, 1 bit shift mode and no byte swap\n");
}
#endif

void platform_85xx_restart(char *cmd)
{
    char *vaddr = NULL;
    volatile unsigned *reg_addr = NULL;
    phys_addr_t paddr = 0;
    uint32_t data = 0;

    if (silkworm_cpld_warm_reset_flag == 0x1) {
        if (silkworm.board == SWBD89) {
            paddr = 0xF8000000UL;
            /* remap the CPLD into our memory space */
            vaddr = ioremap_nocache(paddr, 0x400);

        } else if (silkworm.board == SWBD68) {
            /* map the local control space */
            paddr = 0x1F8000000ULL;
            vaddr = ioremap_nocache(paddr, 0x100);
        }

        if (NULL != vaddr)
        {
            reg_addr = (volatile unsigned *)(vaddr + 0x18);
            data = in_be32(reg_addr);
            data |= 0x00000040;
            out_be32(reg_addr, data);

            printk(KERN_EMERG"\n Kernel: Performing BP warm reset---\n");
            data = 0;
            reg_addr = (volatile unsigned *)(vaddr + 0x50);
            data = in_be32(reg_addr);
            data |= 0x00000050;
            out_be32(reg_addr, data);

        } else {
            printk(KERN_EMERG"\n CPLD mmap for warm reset Failed \n");
        }
        /* should not reach here : for safety reseting the flag*/
        printk(KERN_EMERG"\n BP warm reset did not work, doing machine restart \n");
        mpc85xx_restart(cmd);
        silkworm_cpld_warm_reset_flag = 0;
    } else {
        mpc85xx_restart(cmd);
    }
}

/* ************************************************************************ */
void __init
platform_init(unsigned long r3, unsigned long r4, unsigned long r5,
              unsigned long r6, unsigned long r7)
{
	void *sin_data = NULL;
	extern void sin_copy_raw_atboot( void *);

	/* parse_bootinfo must always be called first */
	parse_bootinfo(find_bootinfo());

	/*
	 * If we were passed in a board information, copy it into the
	 * residual data area.
	 */
	if (r3) {
		memcpy((void *) __res, (void *) (r3 + KERNELBASE),
				sizeof (bd_t));
		/* Get the pointer to SIN */
		sin_data =  (void  *)(r3 + KERNELBASE + sizeof(bd_t));

	}

#ifdef CONFIG_SERIAL_TEXT_DEBUG
	{
		bd_t *binfo = (bd_t *) __res;
		struct uart_port p;

		/* Use the last TLB entry to map CCSRBAR to allow access to DUART regs */
		settlbcam(num_tlbcam_entries - 1, binfo->bi_immr_base,
				binfo->bi_immr_base, MPC85xx_CCSRBAR_SIZE, _PAGE_IO, 0);

		memset(&p, 0, sizeof (p));
		p.iotype = SERIAL_IO_MEM;
		p.membase = (void *) binfo->bi_immr_base + MPC85xx_UART0_OFFSET;
		p.uartclk = binfo->bi_busfreq;

		gen550_init(0, &p);

		memset(&p, 0, sizeof (p));
		p.iotype = SERIAL_IO_MEM;
		p.membase = (void *) binfo->bi_immr_base + MPC85xx_UART1_OFFSET;
		p.uartclk = binfo->bi_busfreq;

		gen550_init(1, &p);
	}
#endif

#if defined(CONFIG_BLK_DEV_INITRD)
	/*
	 * If the init RAM disk has been configured in, and there's a valid
	 * starting address for it, set it up.
	 */
	if (r4) {
		initrd_start = r4 + KERNELBASE;
		initrd_end = r5 + KERNELBASE;
	}
#endif /* CONFIG_BLK_DEV_INITRD */

	/* Copy the kernel command line arguments to a safe place. */

	if (r6) {
		*(char *) (r7 + KERNELBASE) = 0;
		strcpy(cmd_line, (char *) (r6 + KERNELBASE));
	}

	identify_ppc_sys_by_id(mfspr(SPRN_SVR));

	/* Initialize the SIN data from what is passed by bootloader */
	if (sin_data) {
		sin_copy_raw_atboot(sin_data);
	}

	silkworm85xx_setup_pci_phy_base();

	/* setup the PowerPC module struct */
	ppc_md.setup_arch = silkworm85xx_setup_arch;
	ppc_md.show_cpuinfo = silkworm85xx_show_cpuinfo;

	ppc_md.init_IRQ = silkworm85xx_init_IRQ;
	ppc_md.get_irq = openpic_get_irq;

	if ((silkworm.board == SWBD89) ||
            (silkworm.board == SWBD68)) {
	    ppc_md.restart = platform_85xx_restart;
	} else {
	    ppc_md.restart = mpc85xx_restart;
	}
	ppc_md.power_off = mpc85xx_power_off;
	ppc_md.halt = mpc85xx_halt;

	ppc_md.find_end_of_memory = mpc85xx_find_end_of_memory;

	ppc_md.calibrate_decr = mpc85xx_calibrate_decr;

#if defined(CONFIG_SERIAL_8250) && defined(CONFIG_SERIAL_TEXT_DEBUG)
	ppc_md.progress = gen550_progress;
#endif /* CONFIG_SERIAL_8250 && CONFIG_SERIAL_TEXT_DEBUG */
#if defined(CONFIG_SERIAL_8250) && defined(CONFIG_KGDB)
	ppc_md.early_serial_map = mpc85xx_early_serial_map;
#endif	/* CONFIG_SERIAL_8250 && CONFIG_KGDB */

#if defined(CONFIG_BRCD_CPLD_IDE)
	switch (silkworm.board) {
	case SWBD64:
	case SWBD55:
	case SWBD62:
	case SWBD63:
	case SWBD67:
	case SWBD68:
	case SWBD76:
	case SWBD77:
	case SWBD79: /* Spear CP */
	case SWBD89: /* Striker */
	case SWBD133:
		ppc_ide_md.ide_init_hwif = cpld_scimitarfsl_ide_init_hwif_ports;
		break;
	default:
		printk(KERN_WARNING "cpld_ide_init_hwif_ports: The platform is "
				"not supported\n");
		break;
	}
#endif

	/* We need to supply memhole routine defined in silkworm_feature.c */
	switch (silkworm.board) {
	case SWBD62:
	case SWBD67:
	case SWBD68:
	case SWBD76: /* what is this mem hole feature for ask Neeraj */
	case SWBD77:
	case SWBD79: /* Spear CP */
	case SWBD89: /* Striker*/
		ppc_md.feature_call = ppc44x_do_feature_call;
		break;

	case SWBD64:            // Thor has power off
		ppc_md.power_off = cpld_power_off;
		break;
	case SWBD133:
		ppc_md.power_off = cpld_power_off_odin;
		break;

	default:
		/* This BD does not support the buggy Condor :-) */
		break;
	}

	/* Setup the watchdog */
#ifdef CONFIG_BOOKE_WDT
/* Look for wdt= option on command line */
	if (strstr(cmd_line, "wdt=")) {
		int valid_wdt = 0;
		char *p, *q;
		for (q = cmd_line; (p = strstr(q, "wdt=")) != 0;) {
			q = p + 4;
			if (p > cmd_line && p[-1] != ' ')
				continue;
			sw_wdt_period = simple_strtoul(q, &q, 0);
			valid_wdt = 1;
			++q;
		}
		sw_wdt_enable = booke_wdt_enabled =  valid_wdt;
	}

	/* Soft watchdog */
	ppc_md.heartbeat = ppc_booke_sw_wdt_heartbeat;
	ppc_md.heartbeat_count = 0;
#endif

	if (ppc_md.progress)
		ppc_md.progress("silkworm85xx_init(): exit", 0);

	return;
}

/*
 * This routine disables the USB intrrupt.
 * returns 0 on success -1 otherwise
 */
int silkworm_mask_usb_intr()
{
	unsigned char  val;

	switch (silkworm.board) {
	case SWBD62:
	case SWBD64:
	case SWBD79: /* Spear CP */
	case SWBD67:
	case SWBD68:
	case SWBD89:
    case SWBD77:
		if (cpld_read_register( SILKWORM_CPLD_USB_INTR_MASK_REG, &val)) {
			/* Error in read */
			printk(KERN_ERR "silkfsl: Unable to read CPLD USB reg\n");
			return -1;
		}

		/* Mask USB intrrupt set the bit zero */
		val |= 0x1;

		if (cpld_write_register( SILKWORM_CPLD_USB_INTR_MASK_REG, val)) {
			/* Error in read */
			printk(KERN_ERR "silkfsl: Unable to mask CPLD USB reg\n");
			return -1;
		}
		break;

	case SWBD76:
		if (cpld_read_register( ELARA_CPLD_USB_INTR_MASK_REG, &val)) {
			/* Error in read */
			printk(KERN_ERR "silkfsl: Unable to read CPLD USB reg\n");
			return -1;
		}

		/* Mask USB intrrupt set the bit zero */
		val |= 0x1;

		if (cpld_write_register( ELARA_CPLD_USB_INTR_MASK_REG, val)) {
			/* Error in read */
			printk(KERN_ERR "silkfsl: Unable to mask CPLD USB reg\n");
			return -1;
		}
		break;

	case SWBD133:
		    if (cpld_read_register( ODIN_CPLD_USB_INTR_MASK_REG, &val)) {
                        /* Error in read */
                        printk(KERN_ERR "silkfsl: Unable to read CPLD USB reg\n");
                        return -1;
                }
	 	/* Mask USB intrrupt set the bit  1 */
                val |= 0x4;

                if (cpld_write_register( ODIN_CPLD_USB_INTR_MASK_REG, val)) {
                        /* Error in read */
                        printk(KERN_ERR "silkfsl: Unable to mask CPLD USB reg\n");
                        return -1;
                }
                break;



	default:
		printk(KERN_ERR "silkfsl: CPLD USB not supported BD %d\n",
				silkworm.board);
		return -1;
	}

	return 0;
}

/*
 * This routine enables the USB intrrupt.
 * returns 0 on success -1 otherwise
 */
int silkworm_unmask_usb_intr()
{
	unsigned char  val;

	switch (silkworm.board) {
	case SWBD62:
	
	case SWBD64:
	case SWBD67:
	case SWBD77:
	case SWBD79: /* Spear CP */
		if (cpld_read_register( SILKWORM_CPLD_USB_INTR_MASK_REG, &val)) {
			/* Error in read */
			printk(KERN_ERR "silkfsl: Unable to read CPLD USB reg\n");
			return -1;
		}

		/* Unmask USB intrrupt, zero the first bit */
		val &= 0xFE;

		if (cpld_write_register( SILKWORM_CPLD_USB_INTR_MASK_REG, val)) {
			/* Error in read */
			printk(KERN_ERR "silkfsl: Unable to unmask CPLD USB reg\n");
			return -1;
		}
		break;

	case SWBD76:

		if (cpld_read_register( ELARA_CPLD_USB_INTR_MASK_REG, &val)) {
			/* Error in read */
			printk(KERN_ERR "silkfsl: Unable to read CPLD USB reg\n");
			return -1;
		}

		/* Unmask USB intrrupt, zero the first bit */
		val &= 0xFE;

		if (cpld_write_register( ELARA_CPLD_USB_INTR_MASK_REG, val)) {
			/* Error in read */
			printk(KERN_ERR "silkfsl: Unable to unmask CPLD USB reg\n");
			return -1;
		}
		break;
	
	case SWBD133:
		 if (cpld_read_register( ODIN_CPLD_USB_INTR_MASK_REG, &val)) {
                        /* Error in read */
                        printk(KERN_ERR "silkfsl: Unable to read CPLD USB reg\n");
                        return -1;
                }

                /* Unmask USB intrrupt, zero the first bit */
                val &= 0xFB;

                if (cpld_write_register( ODIN_CPLD_USB_INTR_MASK_REG, val)) {
                        /* Error in read */
                        printk(KERN_ERR "silkfsl: Unable to unmask CPLD USB reg\n");
                        return -1;
                }
                break;


	default:
		printk(KERN_ERR "silkfsl: CPLD USB not supported BD %d\n",
				silkworm.board);
		return -1;
	}

	return 0;
}

int silkworm_get_usb_irqno(int irq)
{
	/* Bootrom should take care of it, BUT... */
	switch (silkworm.board) {
	case SWBD62:
	case SWBD64:
	case SWBD67:
	case SWBD77:
	case SWBD79: /* Spear CP */
	    return (MPC85xx_IRQ_EXT11);
	case SWBD76:
	    return (MPC85xx_IRQ_EXT0);
	case SWBD133: /* Odin */
	    return (MPC85xx_IRQ_EXT5);	
	}

	return irq;
}

EXPORT_SYMBOL(silkworm_get_usb_irqno);
EXPORT_SYMBOL(silkworm_mask_usb_intr);
EXPORT_SYMBOL(silkworm_unmask_usb_intr);
EXPORT_SYMBOL(silkworm_cpld_warm_reset_flag);
